@@ -8,19 +8,15 @@ from amcrest import AmcrestError
 from haffmpeg.camera import CameraMjpeg
 import voluptuous as vol
 
-from homeassistant.components.camera import (
-    CAMERA_SERVICE_SCHEMA,
-    SUPPORT_ON_OFF,
-    SUPPORT_STREAM,
-    Camera,
-)
+from homeassistant.components.camera import SUPPORT_ON_OFF, SUPPORT_STREAM, Camera
 from homeassistant.components.ffmpeg import DATA_FFMPEG
-from homeassistant.const import CONF_NAME, STATE_OFF, STATE_ON
+from homeassistant.const import ATTR_ENTITY_ID, CONF_NAME, STATE_OFF, STATE_ON
 from homeassistant.helpers.aiohttp_client import (
     async_aiohttp_proxy_stream,
     async_aiohttp_proxy_web,
     async_get_clientsession,
 )
+import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
 
 from .const import (
@@ -51,6 +47,28 @@ _SRV_CBW = "set_color_bw"
 _SRV_TOUR_ON = "start_tour"
 _SRV_TOUR_OFF = "stop_tour"
 
+_SRV_PTZ_CTRL = "ptz_control"
+_ATTR_PTZ_TT = "travel_time"
+_ATTR_PTZ_MOV = "movement"
+_MOV = [
+    "zoom_out",
+    "zoom_in",
+    "right",
+    "left",
+    "up",
+    "down",
+    "right_down",
+    "right_up",
+    "left_down",
+    "left_up",
+]
+_ZOOM_ACTIONS = ["ZoomWide", "ZoomTele"]
+_MOVE_1_ACTIONS = ["Right", "Left", "Up", "Down"]
+_MOVE_2_ACTIONS = ["RightDown", "RightUp", "LeftDown", "LeftUp"]
+_ACTION = _ZOOM_ACTIONS + _MOVE_1_ACTIONS + _MOVE_2_ACTIONS
+
+_DEFAULT_TT = 0.2
+
 _ATTR_PRESET = "preset"
 _ATTR_COLOR_BW = "color_bw"
 
@@ -59,24 +77,34 @@ _CBW_AUTO = "auto"
 _CBW_BW = "bw"
 _CBW = [_CBW_COLOR, _CBW_AUTO, _CBW_BW]
 
-_SRV_GOTO_SCHEMA = CAMERA_SERVICE_SCHEMA.extend(
+_SRV_SCHEMA = vol.Schema({vol.Optional(ATTR_ENTITY_ID): cv.comp_entity_ids})
+_SRV_GOTO_SCHEMA = _SRV_SCHEMA.extend(
     {vol.Required(_ATTR_PRESET): vol.All(vol.Coerce(int), vol.Range(min=1))}
 )
-_SRV_CBW_SCHEMA = CAMERA_SERVICE_SCHEMA.extend(
-    {vol.Required(_ATTR_COLOR_BW): vol.In(_CBW)}
+_SRV_CBW_SCHEMA = _SRV_SCHEMA.extend({vol.Required(_ATTR_COLOR_BW): vol.In(_CBW)})
+_SRV_PTZ_SCHEMA = _SRV_SCHEMA.extend(
+    {
+        vol.Required(_ATTR_PTZ_MOV): vol.In(_MOV),
+        vol.Optional(_ATTR_PTZ_TT, default=_DEFAULT_TT): cv.small_float,
+    }
 )
 
 CAMERA_SERVICES = {
-    _SRV_EN_REC: (CAMERA_SERVICE_SCHEMA, "async_enable_recording", ()),
-    _SRV_DS_REC: (CAMERA_SERVICE_SCHEMA, "async_disable_recording", ()),
-    _SRV_EN_AUD: (CAMERA_SERVICE_SCHEMA, "async_enable_audio", ()),
-    _SRV_DS_AUD: (CAMERA_SERVICE_SCHEMA, "async_disable_audio", ()),
-    _SRV_EN_MOT_REC: (CAMERA_SERVICE_SCHEMA, "async_enable_motion_recording", ()),
-    _SRV_DS_MOT_REC: (CAMERA_SERVICE_SCHEMA, "async_disable_motion_recording", ()),
+    _SRV_EN_REC: (_SRV_SCHEMA, "async_enable_recording", ()),
+    _SRV_DS_REC: (_SRV_SCHEMA, "async_disable_recording", ()),
+    _SRV_EN_AUD: (_SRV_SCHEMA, "async_enable_audio", ()),
+    _SRV_DS_AUD: (_SRV_SCHEMA, "async_disable_audio", ()),
+    _SRV_EN_MOT_REC: (_SRV_SCHEMA, "async_enable_motion_recording", ()),
+    _SRV_DS_MOT_REC: (_SRV_SCHEMA, "async_disable_motion_recording", ()),
     _SRV_GOTO: (_SRV_GOTO_SCHEMA, "async_goto_preset", (_ATTR_PRESET,)),
     _SRV_CBW: (_SRV_CBW_SCHEMA, "async_set_color_bw", (_ATTR_COLOR_BW,)),
-    _SRV_TOUR_ON: (CAMERA_SERVICE_SCHEMA, "async_start_tour", ()),
-    _SRV_TOUR_OFF: (CAMERA_SERVICE_SCHEMA, "async_stop_tour", ()),
+    _SRV_TOUR_ON: (_SRV_SCHEMA, "async_start_tour", ()),
+    _SRV_TOUR_OFF: (_SRV_SCHEMA, "async_stop_tour", ()),
+    _SRV_PTZ_CTRL: (
+        _SRV_PTZ_SCHEMA,
+        "async_ptz_control",
+        (_ATTR_PTZ_MOV, _ATTR_PTZ_TT),
+    ),
 }
 
 _BOOL_TO_STATE = {True: STATE_ON, False: STATE_OFF}
@@ -96,6 +124,10 @@ class CannotSnapshot(Exception):
     """Conditions are not valid for taking a snapshot."""
 
 
+class AmcrestCommandFailed(Exception):
+    """Amcrest camera command did not work."""
+
+
 class AmcrestCam(Camera):
     """An implementation of an Amcrest IP camera."""
 
@@ -159,7 +191,7 @@ class AmcrestCam(Camera):
             # and before initiating shapshot.
             while self._snapshot_task:
                 self._check_snapshot_ok()
-                _LOGGER.debug("Waiting for previous snapshot from %s ...", self._name)
+                _LOGGER.debug("Waiting for previous snapshot from %s", self._name)
                 await self._snapshot_task
             self._check_snapshot_ok()
             # Run snapshot command in separate Task that can't be cancelled so
@@ -198,7 +230,7 @@ class AmcrestCam(Camera):
         # streaming via ffmpeg
 
         streaming_url = self._rtsp_url
-        stream = CameraMjpeg(self._ffmpeg.binary, loop=self.hass.loop)
+        stream = CameraMjpeg(self._ffmpeg.binary)
         await stream.open_camera(streaming_url, extra_cmd=self._ffmpeg_arguments)
 
         try:
@@ -228,7 +260,7 @@ class AmcrestCam(Camera):
         return self._name
 
     @property
-    def device_state_attributes(self):
+    def extra_state_attributes(self):
         """Return the Amcrest-specific camera state attributes."""
         attr = {}
         if self._audio_enabled is not None:
@@ -333,12 +365,12 @@ class AmcrestCam(Camera):
                     self._model = resp.split("=")[-1]
                 else:
                     self._model = "unknown"
-            self.is_streaming = self._api.video_enabled
-            self._is_recording = self._api.record_mode == "Manual"
-            self._motion_detection_enabled = self._api.is_motion_detector_on()
-            self._audio_enabled = self._api.audio_enabled
-            self._motion_recording_enabled = self._api.is_record_on_motion_detection()
-            self._color_bw = _CBW[self._api.day_night_color]
+            self.is_streaming = self._get_video()
+            self._is_recording = self._get_recording()
+            self._motion_detection_enabled = self._get_motion_detection()
+            self._audio_enabled = self._get_audio()
+            self._motion_recording_enabled = self._get_motion_recording()
+            self._color_bw = self._get_color_mode()
             self._rtsp_url = self._api.rtsp_url(typeno=self._resolution)
         except AmcrestError as error:
             log_update_error(_LOGGER, "get", self.name, "camera attributes", error)
@@ -350,11 +382,11 @@ class AmcrestCam(Camera):
 
     def turn_off(self):
         """Turn off camera."""
-        self._enable_video_stream(False)
+        self._enable_video(False)
 
     def turn_on(self):
         """Turn on camera."""
-        self._enable_video_stream(True)
+        self._enable_video(True)
 
     def enable_motion_detection(self):
         """Enable motion detection in the camera."""
@@ -406,30 +438,78 @@ class AmcrestCam(Camera):
         """Call the job and stop camera tour."""
         await self.hass.async_add_executor_job(self._start_tour, False)
 
+    async def async_ptz_control(self, movement, travel_time):
+        """Move or zoom camera in specified direction."""
+        code = _ACTION[_MOV.index(movement)]
+
+        kwargs = {"code": code, "arg1": 0, "arg2": 0, "arg3": 0}
+        if code in _MOVE_1_ACTIONS:
+            kwargs["arg2"] = 1
+        elif code in _MOVE_2_ACTIONS:
+            kwargs["arg1"] = kwargs["arg2"] = 1
+
+        try:
+            await self.hass.async_add_executor_job(
+                partial(self._api.ptz_control_command, action="start", **kwargs)
+            )
+            await asyncio.sleep(travel_time)
+            await self.hass.async_add_executor_job(
+                partial(self._api.ptz_control_command, action="stop", **kwargs)
+            )
+        except AmcrestError as error:
+            log_update_error(
+                _LOGGER, "move", self.name, f"camera PTZ {movement}", error
+            )
+
     # Methods to send commands to Amcrest camera and handle errors
 
-    def _enable_video_stream(self, enable):
+    def _change_setting(self, value, attr, description, action="set"):
+        func = description.replace(" ", "_")
+        description = f"camera {description} to {value}"
+        tries = 3
+        while True:
+            try:
+                getattr(self, f"_set_{func}")(value)
+                new_value = getattr(self, f"_get_{func}")()
+                if new_value != value:
+                    raise AmcrestCommandFailed
+            except (AmcrestError, AmcrestCommandFailed) as error:
+                if tries == 1:
+                    log_update_error(_LOGGER, action, self.name, description, error)
+                    return
+                log_update_error(
+                    _LOGGER, action, self.name, description, error, logging.DEBUG
+                )
+            else:
+                if attr:
+                    setattr(self, attr, new_value)
+                    self.schedule_update_ha_state()
+                return
+            tries -= 1
+
+    def _get_video(self):
+        return self._api.video_enabled
+
+    def _set_video(self, enable):
+        self._api.video_enabled = enable
+
+    def _enable_video(self, enable):
         """Enable or disable camera video stream."""
         # Given the way the camera's state is determined by
         # is_streaming and is_recording, we can't leave
         # recording on if video stream is being turned off.
         if self.is_recording and not enable:
             self._enable_recording(False)
-        try:
-            self._api.video_enabled = enable
-        except AmcrestError as error:
-            log_update_error(
-                _LOGGER,
-                "enable" if enable else "disable",
-                self.name,
-                "camera video stream",
-                error,
-            )
-        else:
-            self.is_streaming = enable
-            self.schedule_update_ha_state()
+        self._change_setting(enable, "is_streaming", "video")
         if self._control_light:
-            self._enable_light(self._audio_enabled or self.is_streaming)
+            self._change_light()
+
+    def _get_recording(self):
+        return self._api.record_mode == "Manual"
+
+    def _set_recording(self, enable):
+        rec_mode = {"Automatic": 0, "Manual": 1}
+        self._api.record_mode = rec_mode["Manual" if enable else "Automatic"]
 
     def _enable_recording(self, enable):
         """Turn recording on or off."""
@@ -437,88 +517,56 @@ class AmcrestCam(Camera):
         # is_streaming and is_recording, we can't leave
         # video stream off if recording is being turned on.
         if not self.is_streaming and enable:
-            self._enable_video_stream(True)
-        rec_mode = {"Automatic": 0, "Manual": 1}
-        try:
-            self._api.record_mode = rec_mode["Manual" if enable else "Automatic"]
-        except AmcrestError as error:
-            log_update_error(
-                _LOGGER,
-                "enable" if enable else "disable",
-                self.name,
-                "camera recording",
-                error,
-            )
-        else:
-            self._is_recording = enable
-            self.schedule_update_ha_state()
+            self._enable_video(True)
+        self._change_setting(enable, "_is_recording", "recording")
+
+    def _get_motion_detection(self):
+        return self._api.is_motion_detector_on()
+
+    def _set_motion_detection(self, enable):
+        self._api.motion_detection = str(enable).lower()
 
     def _enable_motion_detection(self, enable):
         """Enable or disable motion detection."""
-        try:
-            self._api.motion_detection = str(enable).lower()
-        except AmcrestError as error:
-            log_update_error(
-                _LOGGER,
-                "enable" if enable else "disable",
-                self.name,
-                "camera motion detection",
-                error,
-            )
-        else:
-            self._motion_detection_enabled = enable
-            self.schedule_update_ha_state()
+        self._change_setting(enable, "_motion_detection_enabled", "motion detection")
+
+    def _get_audio(self):
+        return self._api.audio_enabled
+
+    def _set_audio(self, enable):
+        self._api.audio_enabled = enable
 
     def _enable_audio(self, enable):
         """Enable or disable audio stream."""
-        try:
-            self._api.audio_enabled = enable
-        except AmcrestError as error:
-            log_update_error(
-                _LOGGER,
-                "enable" if enable else "disable",
-                self.name,
-                "camera audio stream",
-                error,
-            )
-        else:
-            self._audio_enabled = enable
-            self.schedule_update_ha_state()
+        self._change_setting(enable, "_audio_enabled", "audio")
         if self._control_light:
-            self._enable_light(self._audio_enabled or self.is_streaming)
+            self._change_light()
 
-    def _enable_light(self, enable):
+    def _get_indicator_light(self):
+        return "true" in self._api.command(
+            "configManager.cgi?action=getConfig&name=LightGlobal"
+        ).content.decode("utf-8")
+
+    def _set_indicator_light(self, enable):
+        self._api.command(
+            f"configManager.cgi?action=setConfig&LightGlobal[0].Enable={str(enable).lower()}"
+        )
+
+    def _change_light(self):
         """Enable or disable indicator light."""
-        try:
-            self._api.command(
-                "configManager.cgi?action=setConfig&LightGlobal[0].Enable={}".format(
-                    str(enable).lower()
-                )
-            )
-        except AmcrestError as error:
-            log_update_error(
-                _LOGGER,
-                "enable" if enable else "disable",
-                self.name,
-                "indicator light",
-                error,
-            )
+        self._change_setting(
+            self._audio_enabled or self.is_streaming, None, "indicator light"
+        )
+
+    def _get_motion_recording(self):
+        return self._api.is_record_on_motion_detection()
+
+    def _set_motion_recording(self, enable):
+        self._api.motion_recording = str(enable).lower()
 
     def _enable_motion_recording(self, enable):
         """Enable or disable motion recording."""
-        try:
-            self._api.motion_recording = str(enable).lower()
-        except AmcrestError as error:
-            log_update_error(
-                _LOGGER,
-                "enable" if enable else "disable",
-                self.name,
-                "camera motion recording",
-                error,
-            )
-        else:
-            self._motion_recording_enabled = enable
-            self.schedule_update_ha_state()
+        self._change_setting(enable, "_motion_recording_enabled", "motion recording")
 
     def _goto_preset(self, preset):
         """Move camera position and zoom to preset."""
@@ -529,17 +577,15 @@ class AmcrestCam(Camera):
                 _LOGGER, "move", self.name, f"camera to preset {preset}", error
             )
 
+    def _get_color_mode(self):
+        return _CBW[self._api.day_night_color]
+
+    def _set_color_mode(self, cbw):
+        self._api.day_night_color = _CBW.index(cbw)
+
     def _set_color_bw(self, cbw):
         """Set camera color mode."""
-        try:
-            self._api.day_night_color = _CBW.index(cbw)
-        except AmcrestError as error:
-            log_update_error(
-                _LOGGER, "set", self.name, f"camera color mode to {cbw}", error
-            )
-        else:
-            self._color_bw = cbw
-            self.schedule_update_ha_state()
+        self._change_setting(cbw, "_color_bw", "color mode")
 
     def _start_tour(self, start):
         """Start camera tour."""
