@@ -1,39 +1,23 @@
 """Support for the Brother service."""
+from homeassistant.components.sensor import SensorEntity
 from homeassistant.const import DEVICE_CLASS_TIMESTAMP
 from homeassistant.helpers.update_coordinator import CoordinatorEntity
 
 from .const import (
-    ATTR_BLACK_DRUM_COUNTER,
-    ATTR_BLACK_DRUM_REMAINING_LIFE,
-    ATTR_BLACK_DRUM_REMAINING_PAGES,
-    ATTR_CYAN_DRUM_COUNTER,
-    ATTR_CYAN_DRUM_REMAINING_LIFE,
-    ATTR_CYAN_DRUM_REMAINING_PAGES,
-    ATTR_DRUM_COUNTER,
-    ATTR_DRUM_REMAINING_LIFE,
-    ATTR_DRUM_REMAINING_PAGES,
     ATTR_ENABLED,
     ATTR_ICON,
     ATTR_LABEL,
-    ATTR_MAGENTA_DRUM_COUNTER,
-    ATTR_MAGENTA_DRUM_REMAINING_LIFE,
-    ATTR_MAGENTA_DRUM_REMAINING_PAGES,
     ATTR_MANUFACTURER,
     ATTR_UNIT,
     ATTR_UPTIME,
-    ATTR_YELLOW_DRUM_COUNTER,
-    ATTR_YELLOW_DRUM_REMAINING_LIFE,
-    ATTR_YELLOW_DRUM_REMAINING_PAGES,
+    ATTRS_MAP,
     DATA_CONFIG_ENTRY,
     DOMAIN,
     SENSOR_TYPES,
 )
 
 ATTR_COUNTER = "counter"
-ATTR_FIRMWARE = "firmware"
-ATTR_MODEL = "model"
 ATTR_REMAINING_PAGES = "remaining_pages"
-ATTR_SERIAL = "serial"
 
 
 async def async_setup_entry(hass, config_entry, async_add_entities):
@@ -43,11 +27,11 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
     sensors = []
 
     device_info = {
-        "identifiers": {(DOMAIN, coordinator.data[ATTR_SERIAL])},
-        "name": coordinator.data[ATTR_MODEL],
+        "identifiers": {(DOMAIN, coordinator.data.serial)},
+        "name": coordinator.data.model,
         "manufacturer": ATTR_MANUFACTURER,
-        "model": coordinator.data[ATTR_MODEL],
-        "sw_version": coordinator.data.get(ATTR_FIRMWARE),
+        "model": coordinator.data.model,
+        "sw_version": getattr(coordinator.data, "firmware", None),
     }
 
     for sensor in SENSOR_TYPES:
@@ -56,14 +40,14 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
     async_add_entities(sensors, False)
 
 
-class BrotherPrinterSensor(CoordinatorEntity):
+class BrotherPrinterSensor(CoordinatorEntity, SensorEntity):
     """Define an Brother Printer sensor."""
 
     def __init__(self, coordinator, kind, device_info):
         """Initialize."""
         super().__init__(coordinator)
-        self._name = f"{coordinator.data[ATTR_MODEL]} {SENSOR_TYPES[kind][ATTR_LABEL]}"
-        self._unique_id = f"{coordinator.data[ATTR_SERIAL].lower()}_{kind}"
+        self._name = f"{coordinator.data.model} {SENSOR_TYPES[kind][ATTR_LABEL]}"
+        self._unique_id = f"{coordinator.data.serial.lower()}_{kind}"
         self._device_info = device_info
         self.kind = kind
         self._attrs = {}
@@ -77,8 +61,8 @@ class BrotherPrinterSensor(CoordinatorEntity):
     def state(self):
         """Return the state."""
         if self.kind == ATTR_UPTIME:
-            return self.coordinator.data.get(self.kind).isoformat()
-        return self.coordinator.data.get(self.kind)
+            return getattr(self.coordinator.data, self.kind).isoformat()
+        return getattr(self.coordinator.data, self.kind)
 
     @property
     def device_class(self):
@@ -88,30 +72,14 @@ class BrotherPrinterSensor(CoordinatorEntity):
         return None
 
     @property
-    def device_state_attributes(self):
+    def extra_state_attributes(self):
         """Return the state attributes."""
-        remaining_pages = None
-        drum_counter = None
-        if self.kind == ATTR_DRUM_REMAINING_LIFE:
-            remaining_pages = ATTR_DRUM_REMAINING_PAGES
-            drum_counter = ATTR_DRUM_COUNTER
-        elif self.kind == ATTR_BLACK_DRUM_REMAINING_LIFE:
-            remaining_pages = ATTR_BLACK_DRUM_REMAINING_PAGES
-            drum_counter = ATTR_BLACK_DRUM_COUNTER
-        elif self.kind == ATTR_CYAN_DRUM_REMAINING_LIFE:
-            remaining_pages = ATTR_CYAN_DRUM_REMAINING_PAGES
-            drum_counter = ATTR_CYAN_DRUM_COUNTER
-        elif self.kind == ATTR_MAGENTA_DRUM_REMAINING_LIFE:
-            remaining_pages = ATTR_MAGENTA_DRUM_REMAINING_PAGES
-            drum_counter = ATTR_MAGENTA_DRUM_COUNTER
-        elif self.kind == ATTR_YELLOW_DRUM_REMAINING_LIFE:
-            remaining_pages = ATTR_YELLOW_DRUM_REMAINING_PAGES
-            drum_counter = ATTR_YELLOW_DRUM_COUNTER
+        remaining_pages, drum_counter = ATTRS_MAP.get(self.kind, (None, None))
         if remaining_pages and drum_counter:
-            self._attrs[ATTR_REMAINING_PAGES] = self.coordinator.data.get(
-                remaining_pages
+            self._attrs[ATTR_REMAINING_PAGES] = getattr(
+                self.coordinator.data, remaining_pages
             )
-            self._attrs[ATTR_COUNTER] = self.coordinator.data.get(drum_counter)
+            self._attrs[ATTR_COUNTER] = getattr(self.coordinator.data, drum_counter)
         return self._attrs
 
     @property
