@@ -20,7 +20,7 @@ from buienradar.constants import (
 )
 import voluptuous as vol
 
-from homeassistant.components.sensor import PLATFORM_SCHEMA
+from homeassistant.components.sensor import PLATFORM_SCHEMA, SensorEntity
 from homeassistant.const import (
     ATTR_ATTRIBUTION,
     CONF_LATITUDE,
@@ -39,7 +39,6 @@ from homeassistant.const import (
 )
 from homeassistant.core import callback
 import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.entity import Entity
 from homeassistant.util import dt as dt_util
 
 from .const import DEFAULT_TIMEFRAME
@@ -236,7 +235,7 @@ async def async_setup_platform(hass, config, async_add_entities, discovery_info=
     await data.schedule_update(1)
 
 
-class BrSensor(Entity):
+class BrSensor(SensorEntity):
     """Representation of an Buienradar sensor."""
 
     def __init__(self, sensor_type, client_name, coordinates):
@@ -272,7 +271,7 @@ class BrSensor(Entity):
             self.async_write_ha_state()
 
     @callback
-    def _load_data(self, data):
+    def _load_data(self, data):  # noqa: C901
         """Load the sensor with relevant data."""
         # Find sensor
 
@@ -309,7 +308,7 @@ class BrSensor(Entity):
                 try:
                     condition = data.get(FORECAST)[fcday].get(CONDITION)
                 except IndexError:
-                    _LOGGER.warning("No forecast for fcday=%s...", fcday)
+                    _LOGGER.warning("No forecast for fcday=%s", fcday)
                     return False
 
                 if condition:
@@ -339,7 +338,7 @@ class BrSensor(Entity):
                         self._state = round(self._state * 3.6, 1)
                     return True
                 except IndexError:
-                    _LOGGER.warning("No forecast for fcday=%s...", fcday)
+                    _LOGGER.warning("No forecast for fcday=%s", fcday)
                     return False
 
             # update all other sensors
@@ -347,7 +346,7 @@ class BrSensor(Entity):
                 self._state = data.get(FORECAST)[fcday].get(self.type[:-3])
                 return True
             except IndexError:
-                _LOGGER.warning("No forecast for fcday=%s...", fcday)
+                _LOGGER.warning("No forecast for fcday=%s", fcday)
                 return False
 
         if self.type == SYMBOL or self.type.startswith(CONDITION):
@@ -430,7 +429,7 @@ class BrSensor(Entity):
         return self._entity_picture
 
     @property
-    def device_state_attributes(self):
+    def extra_state_attributes(self):
         """Return the state attributes."""
         if self.type.startswith(PRECIPITATION_FORECAST):
             result = {ATTR_ATTRIBUTION: self._attribution}
