@@ -5,14 +5,12 @@ import logging
 from hass_nabucasa import Cloud, cloud_api
 from hass_nabucasa.google_report_state import ErrorResponse
 
+from homeassistant.components.google_assistant.const import DOMAIN as GOOGLE_DOMAIN
 from homeassistant.components.google_assistant.helpers import AbstractConfig
-from homeassistant.const import (
-    CLOUD_NEVER_EXPOSED_ENTITIES,
-    EVENT_HOMEASSISTANT_STARTED,
-    HTTP_OK,
-)
+from homeassistant.const import CLOUD_NEVER_EXPOSED_ENTITIES, HTTP_OK
 from homeassistant.core import CoreState, split_entity_id
 from homeassistant.helpers import entity_registry
+from homeassistant.setup import async_setup_component
 
 from .const import (
     CONF_ENTITY_CONFIG,
@@ -87,8 +85,18 @@ class CloudGoogleConfig(AbstractConfig):
     async def async_initialize(self):
         """Perform async initialization of config."""
         await super().async_initialize()
-        # Remove bad data that was there until 0.103.6 - Jan 6, 2020
-        self._store.pop_agent_user_id(self._user)
+
+        if self.enabled and GOOGLE_DOMAIN not in self.hass.config.components:
+            await async_setup_component(self.hass, GOOGLE_DOMAIN, {})
+
+        # Remove old/wrong user agent ids
+        remove_agent_user_ids = []
+        for agent_user_id in self._store.agent_user_ids:
+            if agent_user_id != self.agent_user_id:
+                remove_agent_user_ids.append(agent_user_id)
+
+        for agent_user_id in remove_agent_user_ids:
+            await self.async_disconnect_agent_user(agent_user_id)
 
         self._prefs.async_listen_updates(self._async_prefs_updated)
 
@@ -128,6 +136,11 @@ class CloudGoogleConfig(AbstractConfig):
         """Return Agent User Id to use for query responses."""
         return self._cloud.username
 
+    @property
+    def has_registered_user_agent(self):
+        """Return if we have a Agent User Id registered."""
+        return len(self._store.agent_user_ids) > 0
+
     def get_agent_user_id(self, context):
         """Get agent user ID making request."""
         return self.agent_user_id
@@ -156,6 +169,9 @@ class CloudGoogleConfig(AbstractConfig):
 
     async def _async_prefs_updated(self, prefs):
         """Handle updated preferences."""
+        if self.enabled and GOOGLE_DOMAIN not in self.hass.config.components:
+            await async_setup_component(self.hass, GOOGLE_DOMAIN, {})
+
         if self.should_report_state != self.is_reporting_state:
             if self.should_report_state:
                 self.async_enable_report_state()
@@ -198,17 +214,7 @@ class CloudGoogleConfig(AbstractConfig):
         if not self._should_expose_entity_id(entity_id):
             return
 
-        if self.hass.state == CoreState.running:
-            self.async_schedule_google_sync_all()
+        if self.hass.state != CoreState.running:
             return
 
-        if self._sync_on_started:
-            return
-
-        self._sync_on_started = True
-
-        async def sync_google(_):
-            """Sync entities to Google."""
-            await self.async_sync_entities_all()
-
-        self.hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STARTED, sync_google)
+        self.async_schedule_google_sync_all()
