@@ -1,11 +1,4 @@
-"""
-Starts a service to scan in intervals for new devices.
-
-Will emit EVENT_PLATFORM_DISCOVERED whenever a new service has been discovered.
-
-Knows which components handle certain types, will make sure they are
-loaded before the EVENT_PLATFORM_DISCOVERED is fired.
-"""
+"""Starts a service to scan in intervals for new devices."""
 from datetime import timedelta
 import json
 import logging
@@ -14,7 +7,8 @@ from netdisco.discovery import NetworkDiscovery
 import voluptuous as vol
 
 from homeassistant import config_entries
-from homeassistant.const import EVENT_HOMEASSISTANT_START
+from homeassistant.components import zeroconf
+from homeassistant.const import EVENT_HOMEASSISTANT_STARTED
 from homeassistant.core import callback
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.discovery import async_discover, async_load_platform
@@ -28,7 +22,6 @@ SERVICE_APPLE_TV = "apple_tv"
 SERVICE_DAIKIN = "daikin"
 SERVICE_DLNA_DMR = "dlna_dmr"
 SERVICE_ENIGMA2 = "enigma2"
-SERVICE_FREEBOX = "freebox"
 SERVICE_HASS_IOS_APP = "hass_ios"
 SERVICE_HASSIO = "hassio"
 SERVICE_HEOS = "heos"
@@ -44,35 +37,22 @@ SERVICE_WEMO = "belkin_wemo"
 SERVICE_WINK = "wink"
 SERVICE_XIAOMI_GW = "xiaomi_gw"
 
+# These have custom protocols
 CONFIG_ENTRY_HANDLERS = {
-    SERVICE_DAIKIN: "daikin",
     SERVICE_TELLDUSLIVE: "tellduslive",
+    "logitech_mediaserver": "squeezebox",
 }
 
+# These have no config flows
 SERVICE_HANDLERS = {
-    SERVICE_MOBILE_APP: ("mobile_app", None),
-    SERVICE_HASS_IOS_APP: ("ios", None),
     SERVICE_NETGEAR: ("device_tracker", None),
-    SERVICE_HASSIO: ("hassio", None),
-    SERVICE_APPLE_TV: ("apple_tv", None),
     SERVICE_ENIGMA2: ("media_player", "enigma2"),
-    SERVICE_WINK: ("wink", None),
-    SERVICE_XIAOMI_GW: ("xiaomi_aqara", None),
     SERVICE_SABNZBD: ("sabnzbd", None),
-    SERVICE_SAMSUNG_PRINTER: ("sensor", "syncthru"),
-    SERVICE_KONNECTED: ("konnected", None),
-    SERVICE_OCTOPRINT: ("octoprint", None),
-    SERVICE_FREEBOX: ("freebox", None),
-    SERVICE_YEELIGHT: ("yeelight", None),
     "yamaha": ("media_player", "yamaha"),
-    "logitech_mediaserver": ("media_player", "squeezebox"),
-    "denonavr": ("media_player", "denonavr"),
     "frontier_silicon": ("media_player", "frontier_silicon"),
     "openhome": ("media_player", "openhome"),
     "bose_soundtouch": ("media_player", "soundtouch"),
     "bluesound": ("media_player", "bluesound"),
-    "kodi": ("media_player", "kodi"),
-    "volumio": ("media_player", "volumio"),
     "lg_smart_device": ("media_player", "lg_soundbar"),
     "nanoleaf_aurora": ("light", "nanoleaf"),
 }
@@ -80,18 +60,32 @@ SERVICE_HANDLERS = {
 OPTIONAL_SERVICE_HANDLERS = {SERVICE_DLNA_DMR: ("media_player", "dlna_dmr")}
 
 MIGRATED_SERVICE_HANDLERS = [
+    SERVICE_APPLE_TV,
     "axis",
     "deconz",
+    SERVICE_DAIKIN,
+    "denonavr",
     "esphome",
     "google_cast",
+    SERVICE_HASS_IOS_APP,
+    SERVICE_HASSIO,
     SERVICE_HEOS,
     "harmony",
     "homekit",
     "ikea_tradfri",
+    "kodi",
+    SERVICE_KONNECTED,
+    SERVICE_MOBILE_APP,
+    SERVICE_OCTOPRINT,
     "philips_hue",
+    SERVICE_SAMSUNG_PRINTER,
     "sonos",
     "songpal",
     SERVICE_WEMO,
+    SERVICE_WINK,
+    SERVICE_XIAOMI_GW,
+    "volumio",
+    SERVICE_YEELIGHT,
 ]
 
 DEFAULT_ENABLED = (
@@ -126,9 +120,6 @@ async def async_setup(hass, config):
     netdisco = NetworkDiscovery()
     already_discovered = set()
 
-    # Disable zeroconf logging, it spams
-    logging.getLogger("zeroconf").setLevel(logging.CRITICAL)
-
     if DOMAIN in config:
         # Platforms ignore by config
         ignored_platforms = config[DOMAIN][CONF_IGNORE]
@@ -147,6 +138,8 @@ async def async_setup(hass, config):
                 platform,
             )
 
+    zeroconf_instance = await zeroconf.async_get_instance(hass)
+
     async def new_service_found(service, info):
         """Handle a new service if one is found."""
         if service in MIGRATED_SERVICE_HANDLERS:
@@ -178,7 +171,7 @@ async def async_setup(hass, config):
 
         # We do not know how to handle this service.
         if not comp_plat:
-            logger.info("Unknown service discovered: %s %s", service, info)
+            logger.debug("Unknown service discovered: %s %s", service, info)
             return
 
         logger.info("Found new service: %s %s", service, info)
@@ -193,7 +186,9 @@ async def async_setup(hass, config):
     async def scan_devices(now):
         """Scan for devices."""
         try:
-            results = await hass.async_add_job(_discover, netdisco)
+            results = await hass.async_add_executor_job(
+                _discover, netdisco, zeroconf_instance
+            )
 
             for result in results:
                 hass.async_create_task(new_service_found(*result))
@@ -209,16 +204,16 @@ async def async_setup(hass, config):
         """Schedule the first discovery when Home Assistant starts up."""
         async_track_point_in_utc_time(hass, scan_devices, dt_util.utcnow())
 
-    hass.bus.async_listen_once(EVENT_HOMEASSISTANT_START, schedule_first)
+    hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STARTED, schedule_first)
 
     return True
 
 
-def _discover(netdisco):
+def _discover(netdisco, zeroconf_instance):
     """Discover devices."""
     results = []
     try:
-        netdisco.scan()
+        netdisco.scan(zeroconf_instance=zeroconf_instance)
 
         for disc in netdisco.discover():
             for service in netdisco.get_info(disc):
