@@ -1,4 +1,5 @@
 """Config flow to configure forked-daapd devices."""
+from contextlib import suppress
 import logging
 
 from pyforked_daapd import ForkedDaapdAPI
@@ -9,7 +10,7 @@ from homeassistant.const import CONF_HOST, CONF_NAME, CONF_PASSWORD, CONF_PORT
 from homeassistant.core import callback
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
 
-from .const import (  # pylint:disable=unused-import
+from .const import (
     CONF_LIBRESPOT_JAVA_PORT,
     CONF_MAX_PLAYLISTS,
     CONF_TTS_PAUSE_TIME,
@@ -31,6 +32,7 @@ DATA_SCHEMA_DICT = {
 }
 
 TEST_CONNECTION_ERROR_DICT = {
+    "forbidden": "forbidden",
     "ok": "ok",
     "websocket_not_enabled": "websocket_not_enabled",
     "wrong_host_or_port": "wrong_host_or_port",
@@ -133,7 +135,7 @@ class ForkedDaapdFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
         if user_input is not None:
             # check for any entries with same host, abort if found
             for entry in self._async_current_entries():
-                if entry.data[CONF_HOST] == user_input[CONF_HOST]:
+                if entry.data.get(CONF_HOST) == user_input[CONF_HOST]:
                     return self.async_abort(reason="already_configured")
             validate_result = await self.validate_input(user_input)
             if validate_result[0] == "ok":  # success
@@ -160,12 +162,10 @@ class ForkedDaapdFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
         if discovery_info.get("properties") and discovery_info["properties"].get(
             "Machine Name"
         ):
-            try:
+            with suppress(ValueError):
                 version_num = int(
                     discovery_info["properties"].get("mtd-version", "0").split(".")[0]
                 )
-            except ValueError:
-                pass
         if version_num < 27:
             return self.async_abort(reason="not_forked_daapd")
         await self.async_set_unique_id(discovery_info["properties"]["Machine Name"])
@@ -176,7 +176,8 @@ class ForkedDaapdFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
             if entry.data.get(CONF_HOST) != discovery_info["host"]:
                 continue
             self.hass.config_entries.async_update_entry(
-                entry, title=discovery_info["properties"]["Machine Name"],
+                entry,
+                title=discovery_info["properties"]["Machine Name"],
             )
             return self.async_abort(reason="already_configured")
 
@@ -186,6 +187,5 @@ class ForkedDaapdFlowHandler(config_entries.ConfigFlow, domain=DOMAIN):
             CONF_NAME: discovery_info["properties"]["Machine Name"],
         }
         self.discovery_schema = vol.Schema(fill_in_schema_dict(zeroconf_data))
-        # pylint: disable=no-member # https://github.com/PyCQA/pylint/issues/3167
         self.context.update({"title_placeholders": zeroconf_data})
         return await self.async_step_user()
