@@ -1,12 +1,14 @@
 """Support for Freebox Delta, Revolution and Mini 4K."""
+from __future__ import annotations
+
 import logging
-from typing import Dict
+from typing import Any
 
 from freebox_api.exceptions import InsufficientPermissionsError
 
 from homeassistant.components.switch import SwitchEntity
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.helpers.typing import HomeAssistantType
+from homeassistant.core import HomeAssistant
 
 from .const import DOMAIN
 from .router import FreeboxRouter
@@ -15,7 +17,7 @@ _LOGGER = logging.getLogger(__name__)
 
 
 async def async_setup_entry(
-    hass: HomeAssistantType, entry: ConfigEntry, async_add_entities
+    hass: HomeAssistant, entry: ConfigEntry, async_add_entities
 ) -> None:
     """Set up the switch."""
     router = hass.data[DOMAIN][entry.unique_id]
@@ -48,7 +50,7 @@ class FreeboxWifiSwitch(SwitchEntity):
         return self._state
 
     @property
-    def device_info(self) -> Dict[str, any]:
+    def device_info(self) -> dict[str, Any]:
         """Return the device information."""
         return self._router.device_info
 
