@@ -7,7 +7,6 @@ import voluptuous as vol
 
 from homeassistant import config_entries
 from homeassistant.const import (
-    ATTR_NAME,
     CONF_HOST,
     CONF_NAME,
     CONF_PASSWORD,
@@ -17,8 +16,6 @@ from homeassistant.const import (
 from homeassistant.core import callback
 
 from .base import FritzBoxPhonebook
-
-# pylint:disable=unused-import
 from .const import (
     CONF_PHONEBOOK,
     CONF_PREFIXES,
@@ -28,6 +25,7 @@ from .const import (
     DEFAULT_USERNAME,
     DOMAIN,
     FRITZ_ACTION_GET_INFO,
+    FRITZ_ATTR_NAME,
     FRITZ_ATTR_SERIAL_NUMBER,
     FRITZ_SERVICE_DEVICE_INFO,
     SERIAL_NUMBER,
@@ -119,7 +117,7 @@ class FritzBoxCallMonitorConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         phonebook_info = await self.hass.async_add_executor_job(
             self._fritzbox_phonebook.fph.phonebook_info, phonebook_id
         )
-        return phonebook_info[ATTR_NAME]
+        return phonebook_info[FRITZ_ATTR_NAME]
 
     async def _get_list_of_phonebook_names(self):
         """Return list of names for all available phonebooks."""
