@@ -1,7 +1,7 @@
 """Class to hold all media player accessories."""
 import logging
 
-from pyhap.const import CATEGORY_SWITCH, CATEGORY_TELEVISION
+from pyhap.const import CATEGORY_SWITCH
 
 from homeassistant.components.media_player import (
     ATTR_INPUT_SOURCE,
@@ -42,17 +42,9 @@ from .accessories import TYPES, HomeAccessory
 from .const import (
     ATTR_KEY_NAME,
     CHAR_ACTIVE,
-    CHAR_ACTIVE_IDENTIFIER,
-    CHAR_CONFIGURED_NAME,
-    CHAR_CURRENT_VISIBILITY_STATE,
-    CHAR_IDENTIFIER,
-    CHAR_INPUT_SOURCE_TYPE,
-    CHAR_IS_CONFIGURED,
     CHAR_MUTE,
     CHAR_NAME,
     CHAR_ON,
-    CHAR_REMOTE_KEY,
-    CHAR_SLEEP_DISCOVER_MODE,
     CHAR_VOLUME,
     CHAR_VOLUME_CONTROL_TYPE,
     CHAR_VOLUME_SELECTOR,
@@ -62,43 +54,15 @@ from .const import (
     FEATURE_PLAY_PAUSE,
     FEATURE_PLAY_STOP,
     FEATURE_TOGGLE_MUTE,
-    KEY_ARROW_DOWN,
-    KEY_ARROW_LEFT,
-    KEY_ARROW_RIGHT,
-    KEY_ARROW_UP,
-    KEY_BACK,
-    KEY_EXIT,
-    KEY_FAST_FORWARD,
-    KEY_INFORMATION,
-    KEY_NEXT_TRACK,
     KEY_PLAY_PAUSE,
-    KEY_PREVIOUS_TRACK,
-    KEY_REWIND,
-    KEY_SELECT,
-    SERV_INPUT_SOURCE,
     SERV_SWITCH,
-    SERV_TELEVISION,
     SERV_TELEVISION_SPEAKER,
 )
+from .type_remotes import REMOTE_KEYS, RemoteInputSelectAccessory
 from .util import get_media_player_features
 
 _LOGGER = logging.getLogger(__name__)
 
-MEDIA_PLAYER_KEYS = {
-    0: KEY_REWIND,
-    1: KEY_FAST_FORWARD,
-    2: KEY_NEXT_TRACK,
-    3: KEY_PREVIOUS_TRACK,
-    4: KEY_ARROW_UP,
-    5: KEY_ARROW_DOWN,
-    6: KEY_ARROW_LEFT,
-    7: KEY_ARROW_RIGHT,
-    8: KEY_SELECT,
-    9: KEY_BACK,
-    10: KEY_EXIT,
-    11: KEY_PLAY_PAUSE,
-    15: KEY_INFORMATION,
-}
 
 # Names may not contain special characters
 # or emjoi (/ is a special character for Apple)
@@ -250,22 +214,22 @@ class MediaPlayer(HomeAccessory):
 
 
 @TYPES.register("TelevisionMediaPlayer")
-class TelevisionMediaPlayer(HomeAccessory):
+class TelevisionMediaPlayer(RemoteInputSelectAccessory):
     """Generate a Television Media Player accessory."""
 
     def __init__(self, *args):
-        """Initialize a Switch accessory object."""
-        super().__init__(*args, category=CATEGORY_TELEVISION)
+        """Initialize a Television Media Player accessory object."""
+        super().__init__(
+            SUPPORT_SELECT_SOURCE,
+            ATTR_INPUT_SOURCE,
+            ATTR_INPUT_SOURCE_LIST,
+            *args,
+        )
         state = self.hass.states.get(self.entity_id)
-
-        self.support_select_source = False
-
-        self.sources = []
-
-        self.chars_tv = [CHAR_REMOTE_KEY]
-        self.chars_speaker = []
         features = state.attributes.get(ATTR_SUPPORTED_FEATURES, 0)
 
+        self.chars_speaker = []
+
         self._supports_play_pause = features & (SUPPORT_PLAY | SUPPORT_PAUSE)
         if features & SUPPORT_VOLUME_MUTE or features & SUPPORT_VOLUME_STEP:
             self.chars_speaker.extend(
@@ -274,27 +238,11 @@ class TelevisionMediaPlayer(HomeAccessory):
             if features & SUPPORT_VOLUME_SET:
                 self.chars_speaker.append(CHAR_VOLUME)
 
-        source_list = state.attributes.get(ATTR_INPUT_SOURCE_LIST, [])
-        if source_list and features & SUPPORT_SELECT_SOURCE:
-            self.support_select_source = True
-
-        serv_tv = self.add_preload_service(SERV_TELEVISION, self.chars_tv)
-        self.set_primary_service(serv_tv)
-        serv_tv.configure_char(CHAR_CONFIGURED_NAME, value=self.display_name)
-        serv_tv.configure_char(CHAR_SLEEP_DISCOVER_MODE, value=True)
-        self.char_active = serv_tv.configure_char(
-            CHAR_ACTIVE, setter_callback=self.set_on_off
-        )
-
-        self.char_remote_key = serv_tv.configure_char(
-            CHAR_REMOTE_KEY, setter_callback=self.set_remote_key
-        )
-
         if CHAR_VOLUME_SELECTOR in self.chars_speaker:
             serv_speaker = self.add_preload_service(
                 SERV_TELEVISION_SPEAKER, self.chars_speaker
             )
-            serv_tv.add_linked_service(serv_speaker)
+            self.serv_tv.add_linked_service(serv_speaker)
 
             name = f"{self.display_name} Volume"
             serv_speaker.configure_char(CHAR_NAME, value=name)
@@ -318,25 +266,6 @@ class TelevisionMediaPlayer(HomeAccessory):
                     CHAR_VOLUME, setter_callback=self.set_volume
                 )
 
-        if self.support_select_source:
-            self.sources = source_list
-            self.char_input_source = serv_tv.configure_char(
-                CHAR_ACTIVE_IDENTIFIER, setter_callback=self.set_input_source
-            )
-            for index, source in enumerate(self.sources):
-                serv_input = self.add_preload_service(
-                    SERV_INPUT_SOURCE, [CHAR_IDENTIFIER, CHAR_NAME]
-                )
-                serv_tv.add_linked_service(serv_input)
-                serv_input.configure_char(CHAR_CONFIGURED_NAME, value=source)
-                serv_input.configure_char(CHAR_NAME, value=source)
-                serv_input.configure_char(CHAR_IDENTIFIER, value=index)
-                serv_input.configure_char(CHAR_IS_CONFIGURED, value=True)
-                input_type = 3 if "hdmi" in source.lower() else 0
-                serv_input.configure_char(CHAR_INPUT_SOURCE_TYPE, value=input_type)
-                serv_input.configure_char(CHAR_CURRENT_VISIBILITY_STATE, value=False)
-                _LOGGER.debug("%s: Added source %s", self.entity_id, source)
-
         self.async_update_state(state)
 
     def set_on_off(self, value):
@@ -377,7 +306,7 @@ class TelevisionMediaPlayer(HomeAccessory):
     def set_remote_key(self, value):
         """Send remote key value if call came from HomeKit."""
         _LOGGER.debug("%s: Set remote key to %s", self.entity_id, value)
-        key_name = MEDIA_PLAYER_KEYS.get(value)
+        key_name = REMOTE_KEYS.get(value)
         if key_name is None:
             _LOGGER.warning("%s: Unhandled key press for %s", self.entity_id, value)
             return
@@ -393,12 +322,13 @@ class TelevisionMediaPlayer(HomeAccessory):
                 service = SERVICE_MEDIA_PLAY_PAUSE
             params = {ATTR_ENTITY_ID: self.entity_id}
             self.async_call_service(DOMAIN, service, params)
-        else:
-            # Unhandled keys can be handled by listening to the event bus
-            self.hass.bus.fire(
-                EVENT_HOMEKIT_TV_REMOTE_KEY_PRESSED,
-                {ATTR_KEY_NAME: key_name, ATTR_ENTITY_ID: self.entity_id},
-            )
+            return
+
+        # Unhandled keys can be handled by listening to the event bus
+        self.hass.bus.async_fire(
+            EVENT_HOMEKIT_TV_REMOTE_KEY_PRESSED,
+            {ATTR_KEY_NAME: key_name, ATTR_ENTITY_ID: self.entity_id},
+        )
 
     @callback
     def async_update_state(self, new_state):
@@ -424,18 +354,4 @@ class TelevisionMediaPlayer(HomeAccessory):
             if self.char_mute.value != current_mute_state:
                 self.char_mute.set_value(current_mute_state)
 
-        # Set active input
-        if self.support_select_source and self.sources:
-            source_name = new_state.attributes.get(ATTR_INPUT_SOURCE)
-            _LOGGER.debug("%s: Set current input to %s", self.entity_id, source_name)
-            if source_name in self.sources:
-                index = self.sources.index(source_name)
-                if self.char_input_source.value != index:
-                    self.char_input_source.set_value(index)
-            elif hk_state:
-                _LOGGER.warning(
-                    "%s: Sources out of sync. Restart Home Assistant",
-                    self.entity_id,
-                )
-                if self.char_input_source.value != 0:
-                    self.char_input_source.set_value(0)
+        self._async_update_input_state(hk_state, new_state)
