@@ -253,42 +253,44 @@ class Thermostat(HomeAccessory):
         hvac_mode = state.state
         homekit_hvac_mode = HC_HASS_TO_HOMEKIT[hvac_mode]
 
-        if CHAR_TARGET_HEATING_COOLING in char_values:
-            # Homekit will reset the mode when VIEWING the temp
-            # Ignore it if its the same mode
-            if char_values[CHAR_TARGET_HEATING_COOLING] != homekit_hvac_mode:
-                target_hc = char_values[CHAR_TARGET_HEATING_COOLING]
-                if target_hc not in self.hc_homekit_to_hass:
-                    # If the target heating cooling state we want does not
-                    # exist on the device, we have to sort it out
-                    # based on the the current and target temperature since
-                    # siri will always send HC_HEAT_COOL_AUTO in this case
-                    # and hope for the best.
-                    hc_target_temp = char_values.get(CHAR_TARGET_TEMPERATURE)
-                    hc_current_temp = _get_current_temperature(state, self._unit)
-                    hc_fallback_order = HC_HEAT_COOL_PREFER_HEAT
-                    if (
-                        hc_target_temp is not None
-                        and hc_current_temp is not None
-                        and hc_target_temp < hc_current_temp
-                    ):
-                        hc_fallback_order = HC_HEAT_COOL_PREFER_COOL
-                    for hc_fallback in hc_fallback_order:
-                        if hc_fallback in self.hc_homekit_to_hass:
-                            _LOGGER.debug(
-                                "Siri requested target mode: %s and the device does not support, falling back to %s",
-                                target_hc,
-                                hc_fallback,
-                            )
-                            target_hc = hc_fallback
-                            break
+        # Homekit will reset the mode when VIEWING the temp
+        # Ignore it if its the same mode
+        if (
+            CHAR_TARGET_HEATING_COOLING in char_values
+            and char_values[CHAR_TARGET_HEATING_COOLING] != homekit_hvac_mode
+        ):
+            target_hc = char_values[CHAR_TARGET_HEATING_COOLING]
+            if target_hc not in self.hc_homekit_to_hass:
+                # If the target heating cooling state we want does not
+                # exist on the device, we have to sort it out
+                # based on the the current and target temperature since
+                # siri will always send HC_HEAT_COOL_AUTO in this case
+                # and hope for the best.
+                hc_target_temp = char_values.get(CHAR_TARGET_TEMPERATURE)
+                hc_current_temp = _get_current_temperature(state, self._unit)
+                hc_fallback_order = HC_HEAT_COOL_PREFER_HEAT
+                if (
+                    hc_target_temp is not None
+                    and hc_current_temp is not None
+                    and hc_target_temp < hc_current_temp
+                ):
+                    hc_fallback_order = HC_HEAT_COOL_PREFER_COOL
+                for hc_fallback in hc_fallback_order:
+                    if hc_fallback in self.hc_homekit_to_hass:
+                        _LOGGER.debug(
+                            "Siri requested target mode: %s and the device does not support, falling back to %s",
+                            target_hc,
+                            hc_fallback,
+                        )
+                        target_hc = hc_fallback
+                        break
 
-                service = SERVICE_SET_HVAC_MODE_THERMOSTAT
-                hass_value = self.hc_homekit_to_hass[target_hc]
-                params = {ATTR_HVAC_MODE: hass_value}
-                events.append(
-                    f"{CHAR_TARGET_HEATING_COOLING} to {char_values[CHAR_TARGET_HEATING_COOLING]}"
-                )
+            service = SERVICE_SET_HVAC_MODE_THERMOSTAT
+            hass_value = self.hc_homekit_to_hass[target_hc]
+            params = {ATTR_HVAC_MODE: hass_value}
+            events.append(
+                f"{CHAR_TARGET_HEATING_COOLING} to {char_values[CHAR_TARGET_HEATING_COOLING]}"
+            )
 
         if CHAR_TARGET_TEMPERATURE in char_values:
             hc_target_temp = char_values[CHAR_TARGET_TEMPERATURE]
@@ -462,23 +464,26 @@ class Thermostat(HomeAccessory):
 
         # Update current temperature
         current_temp = _get_current_temperature(new_state, self._unit)
-        if current_temp is not None:
-            if self.char_current_temp.value != current_temp:
-                self.char_current_temp.set_value(current_temp)
+        if current_temp is not None and self.char_current_temp.value != current_temp:
+            self.char_current_temp.set_value(current_temp)
 
         # Update current humidity
         if CHAR_CURRENT_HUMIDITY in self.chars:
             current_humdity = new_state.attributes.get(ATTR_CURRENT_HUMIDITY)
-            if isinstance(current_humdity, (int, float)):
-                if self.char_current_humidity.value != current_humdity:
-                    self.char_current_humidity.set_value(current_humdity)
+            if (
+                isinstance(current_humdity, (int, float))
+                and self.char_current_humidity.value != current_humdity
+            ):
+                self.char_current_humidity.set_value(current_humdity)
 
         # Update target humidity
         if CHAR_TARGET_HUMIDITY in self.chars:
             target_humdity = new_state.attributes.get(ATTR_HUMIDITY)
-            if isinstance(target_humdity, (int, float)):
-                if self.char_target_humidity.value != target_humdity:
-                    self.char_target_humidity.set_value(target_humdity)
+            if (
+                isinstance(target_humdity, (int, float))
+                and self.char_target_humidity.value != target_humdity
+            ):
+                self.char_target_humidity.set_value(target_humdity)
 
         # Update cooling threshold temperature if characteristic exists
         if self.char_cooling_thresh_temp:
@@ -575,9 +580,8 @@ class WaterHeater(HomeAccessory):
         """Change operation mode to value if call came from HomeKit."""
         _LOGGER.debug("%s: Set heat-cool to %d", self.entity_id, value)
         hass_value = HC_HOMEKIT_TO_HASS[value]
-        if hass_value != HVAC_MODE_HEAT:
-            if self.char_target_heat_cool.value != 1:
-                self.char_target_heat_cool.set_value(1)  # Heat
+        if hass_value != HVAC_MODE_HEAT and self.char_target_heat_cool.value != 1:
+            self.char_target_heat_cool.set_value(1)  # Heat
 
     def set_target_temperature(self, value):
         """Set target temperature to value if call came from HomeKit."""
@@ -596,14 +600,18 @@ class WaterHeater(HomeAccessory):
         """Update water_heater state after state change."""
         # Update current and target temperature
         target_temperature = _get_target_temperature(new_state, self._unit)
-        if target_temperature is not None:
-            if target_temperature != self.char_target_temp.value:
-                self.char_target_temp.set_value(target_temperature)
+        if (
+            target_temperature is not None
+            and target_temperature != self.char_target_temp.value
+        ):
+            self.char_target_temp.set_value(target_temperature)
 
         current_temperature = _get_current_temperature(new_state, self._unit)
-        if current_temperature is not None:
-            if current_temperature != self.char_current_temp.value:
-                self.char_current_temp.set_value(current_temperature)
+        if (
+            current_temperature is not None
+            and current_temperature != self.char_current_temp.value
+        ):
+            self.char_current_temp.set_value(current_temperature)
 
         # Update display units
         if self._unit and self._unit in UNIT_HASS_TO_HOMEKIT:
