@@ -1,29 +1,34 @@
 """Light for Shelly."""
-from typing import Optional, Tuple
+from __future__ import annotations
+
+import asyncio
+import logging
 
 from aioshelly import Block
+import async_timeout
 
 from homeassistant.components.light import (
     ATTR_BRIGHTNESS,
     ATTR_COLOR_TEMP,
-    ATTR_HS_COLOR,
-    ATTR_WHITE_VALUE,
-    SUPPORT_BRIGHTNESS,
-    SUPPORT_COLOR,
-    SUPPORT_COLOR_TEMP,
-    SUPPORT_WHITE_VALUE,
+    ATTR_RGB_COLOR,
+    ATTR_RGBW_COLOR,
+    COLOR_MODE_BRIGHTNESS,
+    COLOR_MODE_COLOR_TEMP,
+    COLOR_MODE_ONOFF,
+    COLOR_MODE_RGB,
+    COLOR_MODE_RGBW,
     LightEntity,
+    brightness_supported,
 )
 from homeassistant.core import callback
 from homeassistant.util.color import (
-    color_hs_to_RGB,
-    color_RGB_to_hs,
     color_temperature_kelvin_to_mired,
     color_temperature_mired_to_kelvin,
 )
 
 from . import ShellyDeviceWrapper
 from .const import (
+    AIOSHELLY_DEVICE_TIMEOUT_SEC,
     COAP,
     DATA_CONFIG_ENTRY,
     DOMAIN,
@@ -34,6 +39,8 @@ from .const import (
 from .entity import ShellyBlockEntity
 from .utils import async_remove_shelly_entity
 
+_LOGGER = logging.getLogger(__name__)
+
 
 async def async_setup_entry(hass, config_entry, async_add_entities):
     """Set up lights for device."""
@@ -68,24 +75,25 @@ class ShellyLight(ShellyBlockEntity, LightEntity):
         super().__init__(wrapper, block)
         self.control_result = None
         self.mode_result = None
-        self._supported_features = 0
+        self._supported_color_modes = set()
         self._min_kelvin = KELVIN_MIN_VALUE_WHITE
         self._max_kelvin = KELVIN_MAX_VALUE
 
-        if hasattr(block, "brightness") or hasattr(block, "gain"):
-            self._supported_features |= SUPPORT_BRIGHTNESS
-        if hasattr(block, "colorTemp"):
-            self._supported_features |= SUPPORT_COLOR_TEMP
-        if hasattr(block, "white"):
-            self._supported_features |= SUPPORT_WHITE_VALUE
         if hasattr(block, "red") and hasattr(block, "green") and hasattr(block, "blue"):
-            self._supported_features |= SUPPORT_COLOR
             self._min_kelvin = KELVIN_MIN_VALUE_COLOR
+            if hasattr(block, "white"):
+                self._supported_color_modes.add(COLOR_MODE_RGBW)
+            else:
+                self._supported_color_modes.add(COLOR_MODE_RGB)
 
-    @property
-    def supported_features(self) -> int:
-        """Supported features."""
-        return self._supported_features
+        if hasattr(block, "colorTemp"):
+            self._supported_color_modes.add(COLOR_MODE_COLOR_TEMP)
+
+        if not self._supported_color_modes:
+            if hasattr(block, "brightness") or hasattr(block, "gain"):
+                self._supported_color_modes.add(COLOR_MODE_BRIGHTNESS)
+            else:
+                self._supported_color_modes.add(COLOR_MODE_ONOFF)
 
     @property
     def is_on(self) -> bool:
@@ -96,7 +104,7 @@ class ShellyLight(ShellyBlockEntity, LightEntity):
         return self.block.output
 
     @property
-    def mode(self) -> Optional[str]:
+    def mode(self) -> str | None:
         """Return the color mode of the light."""
         if self.mode_result:
             return self.mode_result["mode"]
@@ -114,35 +122,40 @@ class ShellyLight(ShellyBlockEntity, LightEntity):
         return "white"
 
     @property
-    def brightness(self) -> int:
-        """Brightness of light."""
+    def brightness(self) -> int | None:
+        """Return the brightness of this light between 0..255."""
         if self.mode == "color":
             if self.control_result:
-                brightness = self.control_result["gain"]
+                brightness_pct = self.control_result["gain"]
             else:
-                brightness = self.block.gain
+                brightness_pct = self.block.gain
         else:
             if self.control_result:
-                brightness = self.control_result["brightness"]
+                brightness_pct = self.control_result["brightness"]
             else:
-                brightness = self.block.brightness
-        return int(brightness / 100 * 255)
+                brightness_pct = self.block.brightness
+
+        return round(255 * brightness_pct / 100)
 
     @property
-    def white_value(self) -> int:
-        """White value of light."""
-        if self.control_result:
-            white = self.control_result["white"]
-        else:
-            white = self.block.white
-        return int(white)
+    def color_mode(self) -> str | None:
+        """Return the color mode of the light."""
+        if self.mode == "color":
+            if hasattr(self.block, "white"):
+                return COLOR_MODE_RGBW
+            return COLOR_MODE_RGB
+
+        if hasattr(self.block, "colorTemp"):
+            return COLOR_MODE_COLOR_TEMP
+
+        if hasattr(self.block, "brightness") or hasattr(self.block, "gain"):
+            return COLOR_MODE_BRIGHTNESS
+
+        return COLOR_MODE_ONOFF
 
     @property
-    def hs_color(self) -> Tuple[float, float]:
-        """Return the hue and saturation color value of light."""
-        if self.mode == "white":
-            return color_RGB_to_hs(255, 255, 255)
-
+    def rgb_color(self) -> tuple[int, int, int]:
+        """Return the rgb color value [int, int, int]."""
         if self.control_result:
             red = self.control_result["red"]
             green = self.control_result["green"]
@@ -151,14 +164,21 @@ class ShellyLight(ShellyBlockEntity, LightEntity):
             red = self.block.red
             green = self.block.green
             blue = self.block.blue
-        return color_RGB_to_hs(red, green, blue)
+        return (red, green, blue)
 
     @property
-    def color_temp(self) -> Optional[int]:
-        """Return the CT color value in mireds."""
-        if self.mode == "color":
-            return None
+    def rgbw_color(self) -> tuple[int, int, int, int]:
+        """Return the rgbw color value [int, int, int, int]."""
+        if self.control_result:
+            white = self.control_result["white"]
+        else:
+            white = self.block.white
 
+        return (*self.rgb_color, white)
+
+    @property
+    def color_temp(self) -> int | None:
+        """Return the CT color value in mireds."""
         if self.control_result:
             color_temp = self.control_result["temp"]
         else:
@@ -178,51 +198,79 @@ class ShellyLight(ShellyBlockEntity, LightEntity):
         """Return the warmest color_temp that this light supports."""
         return int(color_temperature_kelvin_to_mired(self._min_kelvin))
 
+    @property
+    def supported_color_modes(self) -> set | None:
+        """Flag supported color modes."""
+        return self._supported_color_modes
+
     async def async_turn_on(self, **kwargs) -> None:
         """Turn on light."""
         if self.block.type == "relay":
-            self.control_result = await self.block.set_state(turn="on")
+            self.control_result = await self.set_state(turn="on")
             self.async_write_ha_state()
             return
 
         set_mode = None
+        supported_color_modes = self._supported_color_modes
         params = {"turn": "on"}
-        if ATTR_BRIGHTNESS in kwargs:
-            tmp_brightness = int(kwargs[ATTR_BRIGHTNESS] / 255 * 100)
+
+        if ATTR_BRIGHTNESS in kwargs and brightness_supported(supported_color_modes):
+            brightness_pct = int(100 * (kwargs[ATTR_BRIGHTNESS] + 1) / 255)
             if hasattr(self.block, "gain"):
-                params["gain"] = tmp_brightness
+                params["gain"] = brightness_pct
             if hasattr(self.block, "brightness"):
-                params["brightness"] = tmp_brightness
-        if ATTR_COLOR_TEMP in kwargs:
+                params["brightness"] = brightness_pct
+
+        if ATTR_COLOR_TEMP in kwargs and COLOR_MODE_COLOR_TEMP in supported_color_modes:
             color_temp = color_temperature_mired_to_kelvin(kwargs[ATTR_COLOR_TEMP])
             color_temp = min(self._max_kelvin, max(self._min_kelvin, color_temp))
             # Color temperature change - used only in white mode, switch device mode to white
             set_mode = "white"
-            params["red"] = params["green"] = params["blue"] = 255
             params["temp"] = int(color_temp)
-        if ATTR_HS_COLOR in kwargs:
-            red, green, blue = color_hs_to_RGB(*kwargs[ATTR_HS_COLOR])
+
+        if ATTR_RGB_COLOR in kwargs and COLOR_MODE_RGB in supported_color_modes:
             # Color channels change - used only in color mode, switch device mode to color
             set_mode = "color"
-            params["red"] = red
-            params["green"] = green
-            params["blue"] = blue
-        if ATTR_WHITE_VALUE in kwargs:
-            # White channel change - used only in color mode, switch device mode device to color
+            (params["red"], params["green"], params["blue"]) = kwargs[ATTR_RGB_COLOR]
+
+        if ATTR_RGBW_COLOR in kwargs and COLOR_MODE_RGBW in supported_color_modes:
+            # Color channels change - used only in color mode, switch device mode to color
             set_mode = "color"
-            params["white"] = int(kwargs[ATTR_WHITE_VALUE])
+            (params["red"], params["green"], params["blue"], params["white"]) = kwargs[
+                ATTR_RGBW_COLOR
+            ]
 
-        if set_mode and self.mode != set_mode:
-            self.mode_result = await self.wrapper.device.switch_light_mode(set_mode)
+        if await self.set_light_mode(set_mode):
+            self.control_result = await self.set_state(**params)
 
-        self.control_result = await self.block.set_state(**params)
         self.async_write_ha_state()
 
     async def async_turn_off(self, **kwargs) -> None:
         """Turn off light."""
-        self.control_result = await self.block.set_state(turn="off")
+        self.control_result = await self.set_state(turn="off")
         self.async_write_ha_state()
 
+    async def set_light_mode(self, set_mode):
+        """Change device mode color/white if mode has changed."""
+        if set_mode is None or self.mode == set_mode:
+            return True
+
+        _LOGGER.debug("Setting light mode for entity %s, mode: %s", self.name, set_mode)
+        try:
+            async with async_timeout.timeout(AIOSHELLY_DEVICE_TIMEOUT_SEC):
+                self.mode_result = await self.wrapper.device.switch_light_mode(set_mode)
+        except (asyncio.TimeoutError, OSError) as err:
+            _LOGGER.error(
+                "Setting light mode for entity %s failed, state: %s, error: %s",
+                self.name,
+                set_mode,
+                repr(err),
+            )
+            self.wrapper.last_update_success = False
+            return False
+
+        return True
+
     @callback
     def _update_callback(self):
         """When device updates, clear control & mode result that overrides state."""
