@@ -1,5 +1,4 @@
 """Support for SimpliSafe alarm control panels."""
-import logging
 import re
 
 from simplipy.errors import SimplipyError
@@ -21,7 +20,7 @@ from simplipy.websocket import (
 from homeassistant.components.alarm_control_panel import (
     FORMAT_NUMBER,
     FORMAT_TEXT,
-    AlarmControlPanel,
+    AlarmControlPanelEntity,
 )
 from homeassistant.components.alarm_control_panel.const import (
     SUPPORT_ALARM_ARM_AWAY,
@@ -50,11 +49,10 @@ from .const import (
     ATTR_VOICE_PROMPT_VOLUME,
     DATA_CLIENT,
     DOMAIN,
+    LOGGER,
     VOLUME_STRING_MAP,
 )
 
-_LOGGER = logging.getLogger(__name__)
-
 ATTR_BATTERY_BACKUP_POWER_LEVEL = "battery_backup_power_level"
 ATTR_GSM_STRENGTH = "gsm_strength"
 ATTR_PIN_NAME = "pin_name"
@@ -72,7 +70,7 @@ async def async_setup_entry(hass, entry, async_add_entities):
     )
 
 
-class SimpliSafeAlarm(SimpliSafeEntity, AlarmControlPanel):
+class SimpliSafeAlarm(SimpliSafeEntity, AlarmControlPanelEntity):
     """Representation of a SimpliSafe alarm."""
 
     def __init__(self, simplisafe, system):
@@ -121,11 +119,11 @@ class SimpliSafeAlarm(SimpliSafeEntity, AlarmControlPanel):
     @property
     def code_format(self):
         """Return one or more digits/characters."""
-        if not self._simplisafe.options.get(CONF_CODE):
+        if not self._simplisafe.config_entry.options.get(CONF_CODE):
             return None
-        if isinstance(self._simplisafe.options[CONF_CODE], str) and re.search(
-            "^\\d+$", self._simplisafe.options[CONF_CODE]
-        ):
+        if isinstance(
+            self._simplisafe.config_entry.options[CONF_CODE], str
+        ) and re.search("^\\d+$", self._simplisafe.config_entry.options[CONF_CODE]):
             return FORMAT_NUMBER
         return FORMAT_TEXT
 
@@ -142,11 +140,11 @@ class SimpliSafeAlarm(SimpliSafeEntity, AlarmControlPanel):
     @callback
     def _is_code_valid(self, code, state):
         """Validate that a code matches the required one."""
-        if not self._simplisafe.options.get(CONF_CODE):
+        if not self._simplisafe.config_entry.options.get(CONF_CODE):
             return True
 
-        if not code or code != self._simplisafe.options[CONF_CODE]:
-            _LOGGER.warning(
+        if not code or code != self._simplisafe.config_entry.options[CONF_CODE]:
+            LOGGER.warning(
                 "Incorrect alarm code entered (target state: %s): %s", state, code
             )
             return False
@@ -161,10 +159,11 @@ class SimpliSafeAlarm(SimpliSafeEntity, AlarmControlPanel):
         try:
             await self._system.set_off()
         except SimplipyError as err:
-            _LOGGER.error('Error while disarming "%s": %s', self._system.name, err)
+            LOGGER.error('Error while disarming "%s": %s', self._system.system_id, err)
             return
 
         self._state = STATE_ALARM_DISARMED
+        self.async_write_ha_state()
 
     async def async_alarm_arm_home(self, code=None):
         """Send arm home command."""
@@ -174,10 +173,13 @@ class SimpliSafeAlarm(SimpliSafeEntity, AlarmControlPanel):
         try:
             await self._system.set_home()
         except SimplipyError as err:
-            _LOGGER.error('Error while arming "%s" (home): %s', self._system.name, err)
+            LOGGER.error(
+                'Error while arming "%s" (home): %s', self._system.system_id, err
+            )
             return
 
         self._state = STATE_ALARM_ARMED_HOME
+        self.async_write_ha_state()
 
     async def async_alarm_arm_away(self, code=None):
         """Send arm away command."""
@@ -187,10 +189,13 @@ class SimpliSafeAlarm(SimpliSafeEntity, AlarmControlPanel):
         try:
             await self._system.set_away()
         except SimplipyError as err:
-            _LOGGER.error('Error while arming "%s" (away): %s', self._system.name, err)
+            LOGGER.error(
+                'Error while arming "%s" (away): %s', self._system.system_id, err
+            )
             return
 
         self._state = STATE_ALARM_ARMING
+        self.async_write_ha_state()
 
     @callback
     def async_update_from_rest_api(self):
@@ -217,6 +222,22 @@ class SimpliSafeAlarm(SimpliSafeEntity, AlarmControlPanel):
                 }
             )
 
+        # Although system state updates are designed the come via the websocket, the
+        # SimpliSafe cloud can sporadically fail to send those updates as expected; so,
+        # just in case, we synchronize the state via the REST API, too:
+        if self._system.state == SystemStates.alarm:
+            self._state = STATE_ALARM_TRIGGERED
+        elif self._system.state == SystemStates.away:
+            self._state = STATE_ALARM_ARMED_AWAY
+        elif self._system.state in (SystemStates.away_count, SystemStates.exit_delay):
+            self._state = STATE_ALARM_ARMING
+        elif self._system.state == SystemStates.home:
+            self._state = STATE_ALARM_ARMED_HOME
+        elif self._system.state == SystemStates.off:
+            self._state = STATE_ALARM_DISARMED
+        else:
+            self._state = None
+
     @callback
     def async_update_from_websocket_event(self, event):
         """Update the entity with the provided websocket API event data."""
