@@ -15,6 +15,7 @@ tokens are expired. Alternatively, a Stream can be configured with keepalive
 to always keep workers active.
 """
 import logging
+import re
 import secrets
 import threading
 import time
@@ -38,6 +39,13 @@ from .hls import async_setup_hls
 
 _LOGGER = logging.getLogger(__name__)
 
+STREAM_SOURCE_RE = re.compile("//.*:.*@")
+
+
+def redact_credentials(data):
+    """Redact credentials from string data."""
+    return STREAM_SOURCE_RE.sub("//****:****@", data)
+
 
 def create_stream(hass, stream_source, options=None):
     """Create a stream with the specified identfier based on the source url.
@@ -157,7 +165,7 @@ class Stream:
 
     def check_idle(self):
         """Reset access token if all providers are idle."""
-        if all([p.idle for p in self._outputs.values()]):
+        if all(p.idle for p in self._outputs.values()):
             self.access_token = None
 
     def start(self):
@@ -173,7 +181,7 @@ class Stream:
                 target=self._run_worker,
             )
             self._thread.start()
-            _LOGGER.info("Started stream: %s", self.source)
+            _LOGGER.info("Started stream: %s", redact_credentials(str(self.source)))
 
     def update_source(self, new_source):
         """Restart the stream with a new stream source."""
@@ -239,7 +247,7 @@ class Stream:
             self._thread_quit.set()
             self._thread.join()
             self._thread = None
-            _LOGGER.info("Stopped stream: %s", self.source)
+            _LOGGER.info("Stopped stream: %s", redact_credentials(str(self.source)))
 
     async def async_record(self, video_path, duration=30, lookback=5):
         """Make a .mp4 recording from a provided stream."""
@@ -258,6 +266,7 @@ class Stream:
         recorder.video_path = video_path
 
         self.start()
+        _LOGGER.debug("Started a stream recording of %s seconds", duration)
 
         # Take advantage of lookback
         hls = self.outputs().get("hls")
