@@ -30,18 +30,13 @@ API_GEN_1 = "g1"
 API_GEN_2 = "g2"
 MANUFACTURER = "Subaru Corp."
 
-SUPPORTED_PLATFORMS = [
+PLATFORMS = [
     "sensor",
 ]
 
 ICONS = {
     "Avg Fuel Consumption": "mdi:leaf",
-    "EV Time to Full Charge": "mdi:car-electric",
     "EV Range": "mdi:ev-station",
     "Odometer": "mdi:road-variant",
     "Range": "mdi:gas-station",
-    "Tire Pressure FL": "mdi:gauge",
-    "Tire Pressure FR": "mdi:gauge",
-    "Tire Pressure RL": "mdi:gauge",
-    "Tire Pressure RR": "mdi:gauge",
 }
