@@ -1,7 +1,8 @@
 """Support for Timers."""
-from datetime import timedelta
+from __future__ import annotations
+
+from datetime import datetime, timedelta
 import logging
-import typing
 
 import voluptuous as vol
 
@@ -12,7 +13,7 @@ from homeassistant.const import (
     CONF_NAME,
     SERVICE_RELOAD,
 )
-from homeassistant.core import callback
+from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers import collection
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.entity_component import EntityComponent
@@ -20,7 +21,7 @@ from homeassistant.helpers.event import async_track_point_in_utc_time
 from homeassistant.helpers.restore_state import RestoreEntity
 import homeassistant.helpers.service
 from homeassistant.helpers.storage import Store
-from homeassistant.helpers.typing import ConfigType, HomeAssistantType, ServiceCallType
+from homeassistant.helpers.typing import ConfigType, ServiceCallType
 import homeassistant.util.dt as dt_util
 
 _LOGGER = logging.getLogger(__name__)
@@ -31,6 +32,7 @@ ENTITY_ID_FORMAT = DOMAIN + ".{}"
 DEFAULT_DURATION = 0
 ATTR_DURATION = "duration"
 ATTR_REMAINING = "remaining"
+ATTR_FINISHES_AT = "finishes_at"
 CONF_DURATION = "duration"
 
 STATUS_IDLE = "idle"
@@ -64,6 +66,13 @@ UPDATE_FIELDS = {
 }
 
 
+def _format_timedelta(delta: timedelta):
+    total_seconds = delta.total_seconds()
+    hours, remainder = divmod(total_seconds, 3600)
+    minutes, seconds = divmod(remainder, 60)
+    return f"{int(hours)}:{int(minutes):02}:{int(seconds):02}"
+
+
 def _none_to_empty_dict(value):
     if value is None:
         return {}
@@ -78,9 +87,9 @@ CONFIG_SCHEMA = vol.Schema(
                 {
                     vol.Optional(CONF_NAME): cv.string,
                     vol.Optional(CONF_ICON): cv.icon,
-                    vol.Optional(
-                        CONF_DURATION, default=DEFAULT_DURATION
-                    ): cv.time_period,
+                    vol.Optional(CONF_DURATION, default=DEFAULT_DURATION): vol.All(
+                        cv.time_period, _format_timedelta
+                    ),
                 },
             )
         )
@@ -91,7 +100,7 @@ CONFIG_SCHEMA = vol.Schema(
 RELOAD_SERVICE_SCHEMA = vol.Schema({})
 
 
-async def async_setup(hass: HomeAssistantType, config: ConfigType) -> bool:
+async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up an input select."""
     component = EntityComponent(_LOGGER, DOMAIN, hass)
     id_manager = collection.IDManager()
@@ -99,8 +108,8 @@ async def async_setup(hass: HomeAssistantType, config: ConfigType) -> bool:
     yaml_collection = collection.YamlCollection(
         logging.getLogger(f"{__name__}.yaml_collection"), id_manager
     )
-    collection.attach_entity_component_collection(
-        component, yaml_collection, Timer.from_yaml
+    collection.sync_entity_lifecycle(
+        hass, DOMAIN, DOMAIN, component, yaml_collection, Timer.from_yaml
     )
 
     storage_collection = TimerStorageCollection(
@@ -108,7 +117,9 @@ async def async_setup(hass: HomeAssistantType, config: ConfigType) -> bool:
         logging.getLogger(f"{__name__}.storage_collection"),
         id_manager,
     )
-    collection.attach_entity_component_collection(component, storage_collection, Timer)
+    collection.sync_entity_lifecycle(
+        hass, DOMAIN, DOMAIN, component, storage_collection, Timer
+    )
 
     await yaml_collection.async_load(
         [{CONF_ID: id_, **cfg} for id_, cfg in config.get(DOMAIN, {}).items()]
@@ -119,9 +130,6 @@ async def async_setup(hass: HomeAssistantType, config: ConfigType) -> bool:
         storage_collection, DOMAIN, DOMAIN, CREATE_FIELDS, UPDATE_FIELDS
     ).async_setup(hass)
 
-    collection.attach_entity_registry_cleaner(hass, DOMAIN, DOMAIN, yaml_collection)
-    collection.attach_entity_registry_cleaner(hass, DOMAIN, DOMAIN, storage_collection)
-
     async def reload_service_handler(service_call: ServiceCallType) -> None:
         """Reload yaml entities."""
         conf = await component.async_prepare_reload(skip_reset=True)
@@ -156,40 +164,42 @@ class TimerStorageCollection(collection.StorageCollection):
     CREATE_SCHEMA = vol.Schema(CREATE_FIELDS)
     UPDATE_SCHEMA = vol.Schema(UPDATE_FIELDS)
 
-    async def _process_create_data(self, data: typing.Dict) -> typing.Dict:
+    async def _process_create_data(self, data: dict) -> dict:
         """Validate the config is valid."""
         data = self.CREATE_SCHEMA(data)
         # make duration JSON serializeable
-        data[CONF_DURATION] = str(data[CONF_DURATION])
+        data[CONF_DURATION] = _format_timedelta(data[CONF_DURATION])
         return data
 
     @callback
-    def _get_suggested_id(self, info: typing.Dict) -> str:
+    def _get_suggested_id(self, info: dict) -> str:
         """Suggest an ID based on the config."""
         return info[CONF_NAME]
 
-    async def _update_data(self, data: dict, update_data: typing.Dict) -> typing.Dict:
+    async def _update_data(self, data: dict, update_data: dict) -> dict:
         """Return a new updated data object."""
         data = {**data, **self.UPDATE_SCHEMA(update_data)}
         # make duration JSON serializeable
-        data[CONF_DURATION] = str(data[CONF_DURATION])
+        if CONF_DURATION in update_data:
+            data[CONF_DURATION] = _format_timedelta(data[CONF_DURATION])
         return data
 
 
 class Timer(RestoreEntity):
     """Representation of a timer."""
 
-    def __init__(self, config: typing.Dict):
+    def __init__(self, config: dict):
         """Initialize a timer."""
-        self._config = config
-        self.editable = True
-        self._state = STATUS_IDLE
-        self._remaining = config[CONF_DURATION]
-        self._end = None
+        self._config: dict = config
+        self.editable: bool = True
+        self._state: str = STATUS_IDLE
+        self._duration = cv.time_period_str(config[CONF_DURATION])
+        self._remaining: timedelta | None = None
+        self._end: datetime | None = None
         self._listener = None
 
     @classmethod
-    def from_yaml(cls, config: typing.Dict) -> "Timer":
+    def from_yaml(cls, config: dict) -> Timer:
         """Return entity instance initialized from yaml storage."""
         timer = cls(config)
         timer.entity_id = ENTITY_ID_FORMAT.format(config[CONF_ID])
@@ -201,6 +211,11 @@ class Timer(RestoreEntity):
         """If entity should be polled."""
         return False
 
+    @property
+    def force_update(self) -> bool:
+        """Return True to fix restart issues."""
+        return True
+
     @property
     def name(self):
         """Return name of the timer."""
@@ -217,16 +232,21 @@ class Timer(RestoreEntity):
         return self._state
 
     @property
-    def state_attributes(self):
+    def extra_state_attributes(self):
         """Return the state attributes."""
-        return {
-            ATTR_DURATION: str(self._config[CONF_DURATION]),
+        attrs = {
+            ATTR_DURATION: _format_timedelta(self._duration),
             ATTR_EDITABLE: self.editable,
-            ATTR_REMAINING: str(self._remaining),
         }
+        if self._end is not None:
+            attrs[ATTR_FINISHES_AT] = self._end.isoformat()
+        if self._remaining is not None:
+            attrs[ATTR_REMAINING] = _format_timedelta(self._remaining)
+
+        return attrs
 
     @property
-    def unique_id(self) -> typing.Optional[str]:
+    def unique_id(self) -> str | None:
         """Return unique id for the entity."""
         return self._config[CONF_ID]
 
@@ -239,7 +259,8 @@ class Timer(RestoreEntity):
         state = await self.async_get_last_state()
         self._state = state and state.state == state
 
-    async def async_start(self, duration):
+    @callback
+    def async_start(self, duration: timedelta):
         """Start a timer."""
         if self._listener:
             self._listener()
@@ -254,24 +275,28 @@ class Timer(RestoreEntity):
 
         self._state = STATUS_ACTIVE
         start = dt_util.utcnow().replace(microsecond=0)
+
         if self._remaining and newduration is None:
             self._end = start + self._remaining
+
+        elif newduration:
+            self._duration = newduration
+            self._remaining = newduration
+            self._end = start + self._duration
+
         else:
-            if newduration:
-                self._config[CONF_DURATION] = newduration
-                self._remaining = newduration
-            else:
-                self._remaining = self._config[CONF_DURATION]
-            self._end = start + self._config[CONF_DURATION]
+            self._remaining = self._duration
+            self._end = start + self._duration
 
         self.hass.bus.async_fire(event, {"entity_id": self.entity_id})
 
         self._listener = async_track_point_in_utc_time(
-            self.hass, self.async_finished, self._end
+            self.hass, self._async_finished, self._end
         )
         self.async_write_ha_state()
 
-    async def async_pause(self):
+    @callback
+    def async_pause(self):
         """Pause a timer."""
         if self._listener is None:
             return
@@ -284,40 +309,48 @@ class Timer(RestoreEntity):
         self.hass.bus.async_fire(EVENT_TIMER_PAUSED, {"entity_id": self.entity_id})
         self.async_write_ha_state()
 
-    async def async_cancel(self):
+    @callback
+    def async_cancel(self):
         """Cancel a timer."""
         if self._listener:
             self._listener()
             self._listener = None
         self._state = STATUS_IDLE
         self._end = None
-        self._remaining = timedelta()
+        self._remaining = None
         self.hass.bus.async_fire(EVENT_TIMER_CANCELLED, {"entity_id": self.entity_id})
         self.async_write_ha_state()
 
-    async def async_finish(self):
+    @callback
+    def async_finish(self):
+        """Reset and updates the states, fire finished event."""
+        if self._state != STATUS_ACTIVE:
+            return
+
+        if self._listener:
+            self._listener()
+            self._listener = None
+        self._state = STATUS_IDLE
+        self._end = None
+        self._remaining = None
+        self.hass.bus.async_fire(EVENT_TIMER_FINISHED, {"entity_id": self.entity_id})
+        self.async_write_ha_state()
+
+    @callback
+    def _async_finished(self, time):
         """Reset and updates the states, fire finished event."""
         if self._state != STATUS_ACTIVE:
             return
 
         self._listener = None
         self._state = STATUS_IDLE
-        self._remaining = timedelta()
+        self._end = None
+        self._remaining = None
         self.hass.bus.async_fire(EVENT_TIMER_FINISHED, {"entity_id": self.entity_id})
         self.async_write_ha_state()
 
-    async def async_finished(self, time):
-        """Reset and updates the states, fire finished event."""
-        if self._state != STATUS_ACTIVE:
-            return
-
-        self._listener = None
-        self._state = STATUS_IDLE
-        self._remaining = timedelta()
-        self.hass.bus.async_fire(EVENT_TIMER_FINISHED, {"entity_id": self.entity_id})
-        self.async_write_ha_state()
-
-    async def async_update_config(self, config: typing.Dict) -> None:
+    async def async_update_config(self, config: dict) -> None:
         """Handle when the config is updated."""
         self._config = config
+        self._duration = cv.time_period_str(config[CONF_DURATION])
         self.async_write_ha_state()
