@@ -1,8 +1,10 @@
 """Support for Vera devices."""
+from __future__ import annotations
+
 import asyncio
 from collections import defaultdict
 import logging
-from typing import Any, Dict, Generic, List, Optional, Type, TypeVar
+from typing import Any, Generic, TypeVar
 
 import pyvera as veraApi
 from requests.exceptions import RequestException
@@ -153,7 +155,9 @@ async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> b
         controller.stop()
 
     await hass.async_add_executor_job(controller.start)
-    hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, stop_subscription)
+    config_entry.async_on_unload(
+        hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, stop_subscription)
+    )
 
     return True
 
@@ -172,7 +176,7 @@ async def async_unload_entry(hass: HomeAssistant, config_entry: ConfigEntry) ->
     return True
 
 
-def map_vera_device(vera_device: veraApi.VeraDevice, remap: List[int]) -> str:
+def map_vera_device(vera_device: veraApi.VeraDevice, remap: list[int]) -> str:
     """Map vera classes to Home Assistant types."""
 
     type_map = {
@@ -187,7 +191,7 @@ def map_vera_device(vera_device: veraApi.VeraDevice, remap: List[int]) -> str:
         veraApi.VeraSwitch: "switch",
     }
 
-    def map_special_case(instance_class: Type, entity_type: str) -> str:
+    def map_special_case(instance_class: type, entity_type: str) -> str:
         if instance_class is veraApi.VeraSwitch and vera_device.device_id in remap:
             return "light"
         return entity_type
@@ -234,7 +238,9 @@ class VeraDevice(Generic[DeviceType], Entity):
 
     def update(self):
         """Force a refresh from the device if the device is unavailable."""
-        if not self.available:
+        refresh_needed = self.vera_device.should_poll or not self.available
+        _LOGGER.debug("%s: update called (refresh=%s)", self._name, refresh_needed)
+        if refresh_needed:
             self.vera_device.refresh()
 
     @property
@@ -243,12 +249,7 @@ class VeraDevice(Generic[DeviceType], Entity):
         return self._name
 
     @property
-    def should_poll(self) -> bool:
-        """Get polling requirement from vera device."""
-        return self.vera_device.should_poll
-
-    @property
-    def device_state_attributes(self) -> Optional[Dict[str, Any]]:
+    def extra_state_attributes(self) -> dict[str, Any] | None:
         """Return the state attributes of the device."""
         attr = {}
 
