@@ -1,7 +1,9 @@
 """Vizio SmartCast Device support."""
+from __future__ import annotations
+
 from datetime import timedelta
 import logging
-from typing import Any, Callable, Dict, List, Optional, Union
+from typing import Any, Callable
 
 from pyvizio import VizioAsync
 from pyvizio.api.apps import find_app_name
@@ -24,7 +26,7 @@ from homeassistant.const import (
     STATE_OFF,
     STATE_ON,
 )
-from homeassistant.core import callback
+from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers import entity_platform
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
 from homeassistant.helpers.dispatcher import (
@@ -32,7 +34,6 @@ from homeassistant.helpers.dispatcher import (
     async_dispatcher_send,
 )
 from homeassistant.helpers.entity import Entity
-from homeassistant.helpers.typing import HomeAssistantType
 from homeassistant.helpers.update_coordinator import DataUpdateCoordinator
 
 from .const import (
@@ -62,9 +63,9 @@ PARALLEL_UPDATES = 0
 
 
 async def async_setup_entry(
-    hass: HomeAssistantType,
+    hass: HomeAssistant,
     config_entry: ConfigEntry,
-    async_add_entities: Callable[[List[Entity], bool], None],
+    async_add_entities: Callable[[list[Entity], bool], None],
 ) -> None:
     """Set up a Vizio media player entry."""
     host = config_entry.data[CONF_HOST]
@@ -166,7 +167,7 @@ class VizioDevice(MediaPlayerEntity):
         self._model = None
         self._sw_version = None
 
-    def _apps_list(self, apps: List[str]) -> List[str]:
+    def _apps_list(self, apps: list[str]) -> list[str]:
         """Return process apps list based on configured filters."""
         if self._conf_apps.get(CONF_INCLUDE):
             return [app for app in apps if app in self._conf_apps[CONF_INCLUDE]]
@@ -274,7 +275,7 @@ class VizioDevice(MediaPlayerEntity):
         if self._current_app == NO_APP_RUNNING:
             self._current_app = None
 
-    def _get_additional_app_names(self) -> List[Dict[str, Any]]:
+    def _get_additional_app_names(self) -> list[dict[str, Any]]:
         """Return list of additional apps that were included in configuration.yaml."""
         return [
             additional_app["name"] for additional_app in self._additional_app_configs
@@ -282,7 +283,7 @@ class VizioDevice(MediaPlayerEntity):
 
     @staticmethod
     async def _async_send_update_options_signal(
-        hass: HomeAssistantType, config_entry: ConfigEntry
+        hass: HomeAssistant, config_entry: ConfigEntry
     ) -> None:
         """Send update event when Vizio config entry is updated."""
         # Move this method to component level if another entity ever gets added for a single config entry.
@@ -296,7 +297,7 @@ class VizioDevice(MediaPlayerEntity):
         self._conf_apps.update(config_entry.options.get(CONF_APPS, {}))
 
     async def async_update_setting(
-        self, setting_type: str, setting_name: str, new_value: Union[int, str]
+        self, setting_type: str, setting_name: str, new_value: int | str
     ) -> None:
         """Update a setting when update_setting service is called."""
         await self._device.set_setting(
@@ -340,7 +341,7 @@ class VizioDevice(MediaPlayerEntity):
         return self._available
 
     @property
-    def state(self) -> Optional[str]:
+    def state(self) -> str | None:
         """Return the state of the device."""
         return self._state
 
@@ -355,7 +356,7 @@ class VizioDevice(MediaPlayerEntity):
         return self._icon
 
     @property
-    def volume_level(self) -> Optional[float]:
+    def volume_level(self) -> float | None:
         """Return the volume level of the device."""
         return self._volume_level
 
@@ -365,7 +366,7 @@ class VizioDevice(MediaPlayerEntity):
         return self._is_volume_muted
 
     @property
-    def source(self) -> Optional[str]:
+    def source(self) -> str | None:
         """Return current input of the device."""
         if self._current_app is not None and self._current_input in INPUT_APPS:
             return self._current_app
@@ -373,7 +374,7 @@ class VizioDevice(MediaPlayerEntity):
         return self._current_input
 
     @property
-    def source_list(self) -> List[str]:
+    def source_list(self) -> list[str]:
         """Return list of available inputs of the device."""
         # If Smartcast app is in input list, and the app list has been retrieved,
         # show the combination with , otherwise just return inputs
@@ -395,7 +396,7 @@ class VizioDevice(MediaPlayerEntity):
         return self._available_inputs
 
     @property
-    def app_id(self) -> Optional[str]:
+    def app_id(self) -> str | None:
         """Return the ID of the current app if it is unknown by pyvizio."""
         if self._current_app_config and self.app_name == UNKNOWN_APP:
             return {
@@ -407,7 +408,7 @@ class VizioDevice(MediaPlayerEntity):
         return None
 
     @property
-    def app_name(self) -> Optional[str]:
+    def app_name(self) -> str | None:
         """Return the friendly name of the current app."""
         return self._current_app
 
@@ -422,7 +423,7 @@ class VizioDevice(MediaPlayerEntity):
         return self._config_entry.unique_id
 
     @property
-    def device_info(self) -> Dict[str, Any]:
+    def device_info(self) -> dict[str, Any]:
         """Return device registry information."""
         return {
             "identifiers": {(DOMAIN, self._config_entry.unique_id)},
@@ -438,12 +439,12 @@ class VizioDevice(MediaPlayerEntity):
         return self._device_class
 
     @property
-    def sound_mode(self) -> Optional[str]:
+    def sound_mode(self) -> str | None:
         """Name of the current sound mode."""
         return self._current_sound_mode
 
     @property
-    def sound_mode_list(self) -> Optional[List[str]]:
+    def sound_mode_list(self) -> list[str] | None:
         """List of available sound modes."""
         return self._available_sound_modes
 
