@@ -1,14 +1,17 @@
 """Base classes for channels."""
+from __future__ import annotations
 
 import asyncio
 from enum import Enum
 from functools import wraps
 import logging
-from typing import Any, Union
+from typing import Any
 
 import zigpy.exceptions
 
+from homeassistant.const import ATTR_COMMAND
 from homeassistant.core import callback
+from homeassistant.helpers.dispatcher import async_dispatcher_send
 
 from .. import typing as zha_typing
 from ..const import (
@@ -16,11 +19,15 @@ from ..const import (
     ATTR_ATTRIBUTE_ID,
     ATTR_ATTRIBUTE_NAME,
     ATTR_CLUSTER_ID,
-    ATTR_COMMAND,
+    ATTR_TYPE,
     ATTR_UNIQUE_ID,
     ATTR_VALUE,
     CHANNEL_ZDO,
     SIGNAL_ATTR_UPDATED,
+    ZHA_CHANNEL_MSG,
+    ZHA_CHANNEL_MSG_BIND,
+    ZHA_CHANNEL_MSG_CFG_RPT,
+    ZHA_CHANNEL_MSG_DATA,
 )
 from ..helpers import LogMixin, safe_read
 
@@ -56,8 +63,14 @@ def decorate_command(channel, command):
             )
             return result
 
-        except (zigpy.exceptions.DeliveryError, asyncio.TimeoutError) as ex:
-            channel.debug("command failed: %s exception: %s", command.__name__, str(ex))
+        except (zigpy.exceptions.ZigbeeException, asyncio.TimeoutError) as ex:
+            channel.debug(
+                "command failed: '%s' args: '%s' kwargs '%s' exception: '%s'",
+                command.__name__,
+                args,
+                kwds,
+                str(ex),
+            )
             return ex
 
     return wrapper
@@ -75,6 +88,7 @@ class ZigbeeChannel(LogMixin):
     """Base channel for a Zigbee cluster."""
 
     REPORT_CONFIG = ()
+    BIND: bool = True
 
     def __init__(
         self, cluster: zha_typing.ZigpyClusterType, ch_pool: zha_typing.ChannelPoolType
@@ -135,23 +149,48 @@ class ZigbeeChannel(LogMixin):
     async def bind(self):
         """Bind a zigbee cluster.
 
-        This also swallows DeliveryError exceptions that are thrown when
+        This also swallows ZigbeeException exceptions that are thrown when
         devices are unreachable.
         """
         try:
             res = await self.cluster.bind()
             self.debug("bound '%s' cluster: %s", self.cluster.ep_attribute, res[0])
-        except (zigpy.exceptions.DeliveryError, asyncio.TimeoutError) as ex:
+            async_dispatcher_send(
+                self._ch_pool.hass,
+                ZHA_CHANNEL_MSG,
+                {
+                    ATTR_TYPE: ZHA_CHANNEL_MSG_BIND,
+                    ZHA_CHANNEL_MSG_DATA: {
+                        "cluster_name": self.cluster.name,
+                        "cluster_id": self.cluster.cluster_id,
+                        "success": res[0] == 0,
+                    },
+                },
+            )
+        except (zigpy.exceptions.ZigbeeException, asyncio.TimeoutError) as ex:
             self.debug(
                 "Failed to bind '%s' cluster: %s", self.cluster.ep_attribute, str(ex)
             )
+            async_dispatcher_send(
+                self._ch_pool.hass,
+                ZHA_CHANNEL_MSG,
+                {
+                    ATTR_TYPE: ZHA_CHANNEL_MSG_BIND,
+                    ZHA_CHANNEL_MSG_DATA: {
+                        "cluster_name": self.cluster.name,
+                        "cluster_id": self.cluster.cluster_id,
+                        "success": False,
+                    },
+                },
+            )
 
     async def configure_reporting(self) -> None:
         """Configure attribute reporting for a cluster.
 
-        This also swallows DeliveryError exceptions that are thrown when
+        This also swallows ZigbeeException exceptions that are thrown when
         devices are unreachable.
         """
+        event_data = {}
         kwargs = {}
         if self.cluster.cluster_id >= 0xFC00 and self._ch_pool.manufacturer_code:
             kwargs["manufacturer"] = self._ch_pool.manufacturer_code
@@ -160,6 +199,14 @@ class ZigbeeChannel(LogMixin):
             attr = report["attr"]
             attr_name = self.cluster.attributes.get(attr, [attr])[0]
             min_report_int, max_report_int, reportable_change = report["config"]
+            event_data[attr_name] = {
+                "min": min_report_int,
+                "max": max_report_int,
+                "id": attr,
+                "name": attr_name,
+                "change": reportable_change,
+            }
+
             try:
                 res = await self.cluster.configure_reporting(
                     attr, min_report_int, max_report_int, reportable_change, **kwargs
@@ -173,39 +220,67 @@ class ZigbeeChannel(LogMixin):
                     reportable_change,
                     res,
                 )
-            except (zigpy.exceptions.DeliveryError, asyncio.TimeoutError) as ex:
+                event_data[attr_name]["success"] = (
+                    res[0][0].status == 0 or res[0][0].status == 134
+                )
+            except (zigpy.exceptions.ZigbeeException, asyncio.TimeoutError) as ex:
                 self.debug(
                     "failed to set reporting for '%s' attr on '%s' cluster: %s",
                     attr_name,
                     self.cluster.ep_attribute,
                     str(ex),
                 )
+                event_data[attr_name]["success"] = False
 
-    async def async_configure(self):
+        async_dispatcher_send(
+            self._ch_pool.hass,
+            ZHA_CHANNEL_MSG,
+            {
+                ATTR_TYPE: ZHA_CHANNEL_MSG_CFG_RPT,
+                ZHA_CHANNEL_MSG_DATA: {
+                    "cluster_name": self.cluster.name,
+                    "cluster_id": self.cluster.cluster_id,
+                    "attributes": event_data,
+                },
+            },
+        )
+
+    async def async_configure(self) -> None:
         """Set cluster binding and attribute reporting."""
         if not self._ch_pool.skip_configuration:
-            await self.bind()
+            if self.BIND:
+                await self.bind()
             if self.cluster.is_server:
                 await self.configure_reporting()
+            ch_specific_cfg = getattr(self, "async_configure_channel_specific", None)
+            if ch_specific_cfg:
+                await ch_specific_cfg()
             self.debug("finished channel configuration")
         else:
             self.debug("skipping channel configuration")
         self._status = ChannelStatus.CONFIGURED
 
-    async def async_initialize(self, from_cache):
+    async def async_initialize(self, from_cache: bool) -> None:
         """Initialize channel."""
+        if not from_cache and self._ch_pool.skip_configuration:
+            self._status = ChannelStatus.INITIALIZED
+            return
+
         self.debug("initializing channel: from_cache: %s", from_cache)
-        attributes = []
-        for report_config in self._report_config:
-            attributes.append(report_config["attr"])
-        if len(attributes) > 0:
+        attributes = [cfg["attr"] for cfg in self._report_config]
+        if attributes:
             await self.get_attributes(attributes, from_cache=from_cache)
+
+        ch_specific_init = getattr(self, "async_initialize_channel_specific", None)
+        if ch_specific_init:
+            await ch_specific_init(from_cache=from_cache)
+
+        self.debug("finished channel configuration")
         self._status = ChannelStatus.INITIALIZED
 
     @callback
     def cluster_command(self, tsn, command_id, args):
         """Handle commands received to this cluster."""
-        pass
 
     @callback
     def attribute_updated(self, attrid, value):
@@ -220,10 +295,9 @@ class ZigbeeChannel(LogMixin):
     @callback
     def zdo_command(self, *args, **kwargs):
         """Handle ZDO commands on this cluster."""
-        pass
 
     @callback
-    def zha_send_event(self, command: str, args: Union[int, dict]) -> None:
+    def zha_send_event(self, command: str, args: int | dict) -> None:
         """Relay events to hass."""
         self._ch_pool.zha_send_event(
             {
@@ -236,7 +310,6 @@ class ZigbeeChannel(LogMixin):
 
     async def async_update(self):
         """Retrieve latest state from cluster."""
-        pass
 
     async def get_attribute_value(self, attribute, from_cache=True):
         """Get the value for an attribute."""
@@ -248,7 +321,7 @@ class ZigbeeChannel(LogMixin):
             self._cluster,
             [attribute],
             allow_cache=from_cache,
-            only_cache=from_cache,
+            only_cache=from_cache and not self._ch_pool.is_mains_powered,
             manufacturer=manufacturer,
         )
         return result.get(attribute)
@@ -263,19 +336,18 @@ class ZigbeeChannel(LogMixin):
             result, _ = await self.cluster.read_attributes(
                 attributes,
                 allow_cache=from_cache,
-                only_cache=from_cache,
+                only_cache=from_cache and not self._ch_pool.is_mains_powered,
                 manufacturer=manufacturer,
             )
-            results = {attribute: result.get(attribute) for attribute in attributes}
-        except (asyncio.TimeoutError, zigpy.exceptions.DeliveryError) as ex:
+            return result
+        except (asyncio.TimeoutError, zigpy.exceptions.ZigbeeException) as ex:
             self.debug(
                 "failed to get attributes '%s' on '%s' cluster: %s",
                 attributes,
                 self.cluster.ep_attribute,
                 str(ex),
             )
-            results = {}
-        return results
+            return {}
 
     def log(self, level, msg, *args):
         """Log a message."""
@@ -301,7 +373,7 @@ class ZDOChannel(LogMixin):
         self._cluster = cluster
         self._zha_device = device
         self._status = ChannelStatus.CREATED
-        self._unique_id = "{}:{}_ZDO".format(str(device.ieee), device.name)
+        self._unique_id = f"{str(device.ieee)}:{device.name}_ZDO"
         self._cluster.add_listener(self)
 
     @property
@@ -322,12 +394,10 @@ class ZDOChannel(LogMixin):
     @callback
     def device_announce(self, zigpy_device):
         """Device announce handler."""
-        pass
 
     @callback
     def permit_duration(self, duration):
         """Permit handler."""
-        pass
 
     async def async_initialize(self, from_cache):
         """Initialize channel."""
