@@ -1,15 +1,17 @@
 """Triggers."""
+from __future__ import annotations
+
 import asyncio
 import logging
 from types import MappingProxyType
-from typing import Any, Callable, Dict, List, Optional, Union
+from typing import Any, Callable
 
 import voluptuous as vol
 
 from homeassistant.const import CONF_PLATFORM
-from homeassistant.core import CALLBACK_TYPE, callback
+from homeassistant.core import CALLBACK_TYPE, HomeAssistant, callback
 from homeassistant.exceptions import HomeAssistantError
-from homeassistant.helpers.typing import ConfigType, HomeAssistantType
+from homeassistant.helpers.typing import ConfigType
 from homeassistant.loader import IntegrationNotFound, async_get_integration
 
 _PLATFORM_ALIASES = {
@@ -18,9 +20,7 @@ _PLATFORM_ALIASES = {
 }
 
 
-async def _async_get_trigger_platform(
-    hass: HomeAssistantType, config: ConfigType
-) -> Any:
+async def _async_get_trigger_platform(hass: HomeAssistant, config: ConfigType) -> Any:
     platform = config[CONF_PLATFORM]
     for alias, triggers in _PLATFORM_ALIASES.items():
         if platform in triggers:
@@ -39,8 +39,8 @@ async def _async_get_trigger_platform(
 
 
 async def async_validate_trigger_config(
-    hass: HomeAssistantType, trigger_config: List[ConfigType]
-) -> List[ConfigType]:
+    hass: HomeAssistant, trigger_config: list[ConfigType]
+) -> list[ConfigType]:
     """Validate triggers."""
     config = []
     for conf in trigger_config:
@@ -54,15 +54,15 @@ async def async_validate_trigger_config(
 
 
 async def async_initialize_triggers(
-    hass: HomeAssistantType,
-    trigger_config: List[ConfigType],
+    hass: HomeAssistant,
+    trigger_config: list[ConfigType],
     action: Callable,
     domain: str,
     name: str,
     log_cb: Callable,
     home_assistant_start: bool = False,
-    variables: Optional[Union[Dict[str, Any], MappingProxyType]] = None,
-) -> Optional[CALLBACK_TYPE]:
+    variables: dict[str, Any] | MappingProxyType | None = None,
+) -> CALLBACK_TYPE | None:
     """Initialize triggers."""
     info = {
         "domain": domain,
@@ -72,8 +72,9 @@ async def async_initialize_triggers(
     }
 
     triggers = []
-    for conf in trigger_config:
+    for idx, conf in enumerate(trigger_config):
         platform = await _async_get_trigger_platform(hass, conf)
+        info = {**info, "trigger_id": f"{idx}"}
         triggers.append(platform.async_attach_trigger(hass, conf, action, info))
 
     attach_results = await asyncio.gather(*triggers, return_exceptions=True)
